<%@ page contentType="text/javascript" %>
/**
 * Validates that the value is between the specified range or meets the required limit .
 * Receives the rule definition and its scope.
 * scope.$parent will contain the form to enable watch and access to other fields in the same step
 * ctrls will be an array of controllers, first the ffFormController and then the ngModel of the associated input
 *
 * @param {object} rule current rule definition
 * @param {object} scope current input scope
 * @param {object} el current html element
 * @param {array} attr attributes of the html element
 * @param {array} ctrls ctrls[0] = FFFormController ctrls[1]=ngModel
 */

ff.validationRules.range = function (rule, scope, el, attr, ctrls) {

    'use strict';

    var model = ctrls[1];
    var min = isInt(rule.rangemin) ? parseInt(rule.rangemin) : isFloat(rule.rangemin) ? parseFloat(rule.rangemin) : '';
    var max = isInt(rule.rangemax) ? parseInt(rule.rangemax) : isFloat(rule.rangemax) ? parseFloat(rule.rangemax) : '';
    var regexp = new RegExp(/^-{0,1}\d*\.?\,?\d*$/);
    var input = scope.input;

    return function(modelValue, viewValue){
        //A check to see if either an input or checkbox value is empty.
        if (((_.isUndefined(viewValue) || viewValue === null) && input.value === null ) || (typeof viewValue === 'string' && viewValue.trim().length == 0) || (_.isArray(input.value) && input.value.length === 0)) {
            return true;
        }
        //viewValue could be an array of values(in the case of multiple checkboxes/select).
        if (!_.isArray(input.value)) {
            return validateValue(viewValue);
        } else {
            var isValid = true;
            _.each(input.value, function(value) {
               if (!validateValue(value)) {
                   isValid = false;
               }
            });
            return isValid;
        }
    };

    /*
     * Validates to see if the value is a number and if it meets the required range
     * @func
     * @param value the value that is being tested.
     * @returns {boolean} returns whether or not the supplied value is valid
     */
    function validateValue(value) {
        if (!regexp.test(value)) {
            return false;
        }
        value = value.replace(',', '.');
        if(isInt(value)) {
            value = parseInt(value);
        } else if(isFloat(value)) {
            value = parseFloat(value);
        }
        if (!model.$isEmpty(min) && !model.$isEmpty(max)) {
            //min and max value is specified, test if view value is between range
            return (value >= min && value <= max);
        } else if (model.$isEmpty(min) && !model.$isEmpty(max)) {
            //min is not specified and max is specified, validate if view value is ln or eq to max
            return value <= max;
        } else if (!model.$isEmpty(min) && model.$isEmpty(max)) {
            //max is not specified and min is specified, validate if view value is gt or eq to min
            return value >= min;
        }
    }

    function isInt(num) {
        return num === (num|0);
    }

    function isFloat(num) {
        return num !== (num|0);
    }
};