<%@ page contentType="text/javascript" %>
/**
 * Validates that the value is not empty
 * Receives the rule definition and its scope.
 * scope.$parent will contain the form to enable watch and access to other fields in the same step
 * ctrls will be an array of controllers, first the ffFormController and then the ngModel of the associated input
 *
 * @param {object} rule current rule definition
 * @param {object} scope current input scope
 * @param {object} el current html element
 * @param {array} attr attributes of the html element
 * @param {array} ctrls ctrls[0] = FFFormController ctrls[1]=ngModel
 */

ff.validationRules.required = function(rule, scope, el, attr, ctrls) {
    'use strict';
    var model = ctrls[1];
    var formController = ctrls[0].getFormController();
    scope.$watch(function () {
        return scope.form[rule.fieldName].$valid;
    }, function () {
        model.$validate();
    });
    attr.required = true; // force true in case we are on non input element

    return function (modelValue, viewValue) {
        var input = formController[rule.fieldName];
        if (input.$valid && input.$dirty) {
            if (angular.isUndefined(viewValue)) {
                return false;
            }
            return !model.$isEmpty(viewValue);
        } else {
            return model.$isEmpty(viewValue);
        }
    };
};
