<%@
 page
 contentType="text/javascript" %>
<%@ taglib prefix="formfactory" uri="http://www.jahia.org/formfactory/functions" %>
<%--@elvariable id="renderContext" type="org.jahia.services.render.RenderContext"--%>

(function() {
    var simpleDate = function($log, ffTemplateResolver, uibDateParser) {
        var directive = {
            restrict: 'E',
            require: ['^ffController'],
            controller: simpleDateController,
            controllerAs: 'sdc',
            bindToController: true,
            templateUrl: function(el, attrs) {
                return ffTemplateResolver.resolveTemplatePath('${formfactory:addFormFactoryModulePath('/form-factory-definitions/simple-date', renderContext)}', attrs.viewType);
            },
            link: linkFunction
        };

        return directive;

        function linkFunction(scope, el, attr, ctrl) {
            var formController = ctrl[0];
            scope.date = {};
            scope.date.years = [];
            scope.date.months = [];
            //object with days and whether to display as selection.
            scope.date.days = [];
            //convert to boolean in case it is a string
            scope.input.useCurrentDate = (scope.input.useCurrentDate === true || scope.input.useCurrentDate == 'true');

            var m = moment();
            scope.date.placeholder = {
                year: m.year(),
                month: m.month() + 1,
                day: m.date()
            };
            delete m;
            if(formController.isLiveMode()) {
                initDate();
            }
            /**
             * Updates the day selection menu with current month's available days
             * @func
             */
            scope.updateDateModel = function(inputType) {
                //form to dirty because the buttons will not automatically do this
                if(formController.isLiveMode()) {
                    formController.getFormController()[scope.input.name].$setDirty();
                }
                if (inputType === 'year' && (_.isUndefined(scope.date.selectedYear) || !_.isNumber(scope.date.selectedYear))) {
                    resetDate();
                    return;
                }
                var m = moment();

                if (_.isNumber(scope.date.selectedYear)) {
                    //if year is valid just set it.
                    m.year(scope.date.selectedYear);
                } else {
                    //if year has not been set set it to max year
                    scope.date.selectedYear = scope.date.maxYear;
                }

                if (_.isNumber(scope.date.selectedMonth)) {
                    //if month is selected set the month
                    m.set('month', scope.date.selectedMonth-1);
                } else {
                    //if month has not been selected set to current month
                    scope.date.selectedMonth = m.month()+1;
                }

                var days = m.daysInMonth();
                //keep previously selected day in case the date is invalid.
                var latestAvailableDay = {id : scope.date.selectedDay};

                //Update days selection to match days available in current month (and year in case of february in a leap year)
                _.each(scope.date.days, function(day, index) {
                    day.displayable = day.id <= days;
                    if (scope.date.selectedDay === day.id && day.displayable) {
                        m.date(scope.date.selectedDay);
                    } else if (day.displayable) {
                        latestAvailableDay = scope.date.days[index];
                    } else if(scope.date.selectedDay === day.id && !day.displayable) {
                        //if current selected day is no longer valid, set to last day in currently selected month
                        scope.date.selectedDay = latestAvailableDay.id;
                        m.date(scope.date.selectedDay);
                    }
                });

                if(!_.isNumber(scope.date.selectedDay)) {
                    //update the selected day
                    scope.date.selectedDay = latestAvailableDay.id;
                    m.date(scope.date.selectedDay);
                }
                scope.input.value = m.toISOString();
            };

            /**
             * Initializes the year/month/date fields
             * This is only done for live mode.
             * @func
             */
            function initDate() {
                var m = null;
                //choose right moment date to instantiate
                if(scope.input.useCurrentDate) {
                    m = moment();
                    scope.input.value = moment().toISOString();

                } else if (!_.isUndefined(scope.input.value) && moment(scope.input.value).isValid()) {
                    m = moment(scope.input.value);
                }
                //set values if moment date is valid
                if (scope.input.useCurrentDate || (!_.isUndefined(scope.input.value) && moment(scope.input.value).isValid())) {
                    scope.date.selectedYear = m.year();
                    //returned as 0-11
                    scope.date.selectedMonth = m.month() + 1;
                    scope.date.selectedDay = m.date();
                }

                if(!scope.input.useCurrentDate) {
                    if (!moment(scope.input.value).isValid()) {
                        m = moment();
                    }
                    scope.date.minYear = !_.isUndefined(scope.input.minDate) && !_.isEmpty(scope.input.minDate) ? moment(scope.input.minDate).year() : 1970;
                    scope.date.maxYear = !_.isUndefined(scope.input.maxDate) && !_.isEmpty(scope.input.maxDate) ? moment(scope.input.maxDate).year() : moment().year();
                    for (var i = scope.date.minYear; i <= scope.date.maxYear; i++) {
                        scope.date.years.push({
                            id: i
                        });
                    }

                    for (var i = 1; i <= 12; i++) {
                        scope.date.months.push({
                            id: i
                        });
                    }
                    var days = 31;
                    if (_.isNumber(scope.date.selectedMonth)) {
                        days = m.daysInMonth();
                    }
                    for (var i = 1; i <= 31; i++) {
                        scope.date.days.push({
                            id: i,
                            displayable: i <= days
                        });
                    }
                }
            }

            function resetDate() {
                scope.date.selectedMonth = null;
                scope.date.selectedDay = null;
                scope.input.value = null;
            }
            /**
             * Adjusts button size to match the input size
             * @func
             * @return string the appropriate bootstrap button class
             */
            scope.buttonSize = function(size) {
                if (!_.isUndefined(size)) {
                    var btnSize = size.split('-');
                    return btnSize.length > 1 ? 'btn-' + btnSize[1] : '';
                }
                return '';
            };
        }
    };

    angular
            .module('formFactory')
            .directive('ffSimpleDate', ['$log', 'ffTemplateResolver', 'uibDateParser', simpleDate]);

    var simpleDateController = function($scope) {
        var sdc = this;
        //This is controller is used to setup the min and max date for the date picker
        //when it is initialized so that it propagates to the designView.
        if($scope.readOnly) {
            if (!($scope.input.minDate instanceof Date)) {
                //create valid date, as this is the value originating from wizard file.
                $scope.input.minDate = !_.isEmpty($scope.input.minDate) ? moment($scope.input.minDate, 'YYYY').toISOString() : moment(moment().year() - 20, 'YYYY').toISOString();
            }
            if (!($scope.input.maxDate instanceof Date)) {
                //create valid date, as this is the value originating from wizard file.
                $scope.input.maxDate = !_.isEmpty($scope.input.maxDate) ? moment($scope.input.maxDate, 'YYYY').toISOString() : moment(moment().year() + 10, 'YYYY').toISOString();
            }
        }
    };

    simpleDateController.$inject = ['$scope'];

    var validYear = function($log, $parse) {
        var directive = {
            restrict: 'A',
            require: ['^ngModel'],
            link: linkFunction
        };

        return directive;

        function linkFunction(scope, el, attr, ctrl) {
            var model = ctrl[0];
            scope.errorMessages = {
                'angular.ffValidYear.error.minYear': false,
                'angular.ffValidYear.error.maxYear': false
            };

            //Validator for entering in a valid year
            model.$validators['invalidDate_' + scope.input.name] = function (modelValue, viewValue) {
                //reset all error messages
                scope.errorMessages['angular.ffValidYear.error.maxYear'] = false;
                scope.errorMessages['angular.ffValidYear.error.minYear'] = false;

                var valid = true;
                var selectedYear = moment(viewValue).format('Y');
                //validate that the selected year is not less than the acceptable minimum year
                if (!_.isUndefined(scope.date.minYear) && scope.date.minYear > selectedYear) {
                    scope.errorMessages['angular.ffValidYear.error.minYear'] = true;
                    valid = false;
                }
                //validate that the selected year is not greater than the acceptable maximum year
                if (!_.isUndefined(scope.date.maxYear) && scope.date.maxYear < selectedYear) {
                    scope.errorMessages['angular.ffValidYear.error.maxYear'] = true;
                    valid = false;
                }
                return valid;
            };

        }
    };
    angular
            .module('formFactory')
            .directive('ffValidYear', ['$log', '$parse', validYear]);

    var parseValidYear = function($log) {

        var directive = {
            restrict: 'A',
            require: ['^ngModel'],
            link: linkFunction
        };

        return directive;

        function linkFunction(scope, el, attr, ctrls) {
            var model = ctrls[0];
            var viewValue;
            model.$parsers.push(function(value) {
                if (!_.isUndefined(value) && value !== null && value.toString().length > 4) {
                    value = viewValue;
                    model.$setViewValue(value);
                    model.$render();
                } else if (_.isUndefined(value) || value === "" || value === null) {
                    //set view to empty string
                    value = "";
                    viewValue = value;
                } else if (!(isNaN(value) || value <= 0 || value.toString().indexOf('.') != -1)) {
                    value = parseInt(value);
                    viewValue = value;
                    model.$setViewValue(value);
                    model.$render();
                } else {
                    value = viewValue;
                    model.$setViewValue(value);
                    model.$render();
                }
                return value;
            });
        }
    };

    angular
            .module('formFactory')
            .directive('parseValidYear',  ['$log', parseValidYear]);
})();
