'use strict';
/**
 * @namespace ff
 * @global
 */
var ff                  = {};
ff.Filters              = {};
ff.RenderingDirectives  = {};
ff.Serializers          = {};
ff.ValidationDirectives = {};
ff.LogicDirectives      = {};
ff.Factories            = {};
ff.validationRules      = {};
ff.logicRules           = {};
ff.dataProvider         = null; //Specify the file you want to see or null to be in the live mode.
ff.app = angular.module('formFactory', ['i18n', 'formFactory.validation', 'formFactory.logic', 'ngFileUpload',
    'checklist-model', 'formFactory.commonUseFactory', 'formFactory.templateResolver', 'ui.bootstrap',
    'formFactory.dataFactory', 'vcRecaptcha']);
(function () {
	'use strict';
	var input = function ($compile, $log, $window, $timeout) {
		var directive = {
			restrict: 'E',
			require: ['^ffController'],
			scope   : {
				actionData: '=',
				resultData: '=',
				callbackDirectives: '=',
				allCompletedFlag: '=',
				displayTemplates: '=',
				callback: '&'
			},
			link    : linkFunction
		};

		return directive;

		function linkFunction(scope, el, attr, ctrls) {
			var formName = ctrls[0].formName;
			var listenTo = 'ff' + formName + 'NotifyComplete';
			var timeout = 5000;
			//Listen to directive completions
			scope.$on('callbackDone', function(event, callbackName) {
				scope.handleDoneDirectives(callbackName);
			});
			//Listen for global callback completions
			$window.document.addEventListener(listenTo, function(e){
				scope.handleDoneGlobalCallbacks(e.callbackResult.callbackName);
			});
			scope.doneDirectives = [];
			scope.doneGlobalCallbacks = [];

			//Pushes done callback directives into array
			scope.handleDoneDirectives = function(callbackName) {
				if (!_.contains(scope.doneDirectives, callbackName)) {
					scope.doneDirectives.push(callbackName);
					scope.checkIfAllCallbacksAreDone();
				}
			};

			//Pushes done global callbacks into array
			scope.handleDoneGlobalCallbacks = function(callbackName) {
				if (!_.contains(scope.doneGlobalCallbacks, callbackName)) {
					scope.doneGlobalCallbacks.push(callbackName);
					scope.checkIfAllCallbacksAreDone();
				}
			};

			//Checks if all available callbacks have been processed
			scope.checkIfAllCallbacksAreDone = function() {
				if (scope.doneDirectives.length === scope.callbackDirectives.length
					&& scope.doneGlobalCallbacks.length === $window.ffCallbacks[formName].length) {
					$timeout(function () {
						scope.allCompletedFlag = true;
						scope.runRedirectActions();
					});
					$log.info("All callbacks are done!!!");
				}
			};

			//Executes globally defined callbacks
			scope.executeGlobalCallbacks = function() {
				var data = {};
				data.actionData = scope.actionData;
				data.resultData = scope.resultData;
				for (var i in $window.ffCallbacks[formName]) {
					var callbackObj = $window.ffCallbacks[formName][i];
					callbackObj['fcn'].call(callbackObj['ctx'], data, callbackObj.formInfo);
				}
			};

			//If no directive callbacks are registered this fcn will try to process redirects
			scope.runRedirectActions = function() {
				if (scope.callbackDirectives.length === 0 && $window.ffCallbacks[formName].length > 0) {
					ctrls[0].handleRedirectIfPresent(scope.actionData);
				}
			};

			/********************************Functional part of the function*******************************************/

			scope.executeGlobalCallbacks();

			//Compile directive callbacks
			for (var i in scope.callbackDirectives) {
				if (scope.displayTemplates) {
					el.append($compile("<" + scope.callbackDirectives[i]  + " action-data='actionData'></" + scope.callbackDirectives[i] + ">")(scope));
				}
				else {
					el.css("display", "none");
					el.append($compile("<" + scope.callbackDirectives[i]  + " action-data='actionData'></" + scope.callbackDirectives[i] + ">")(scope));
				}
			}

			//Display submission text if global callbacks take too long.
			if (scope.callbackDirectives.length === 0 && $window.ffCallbacks[formName].length > 0) {
				$timeout(function() {
					scope.allCompletedFlag = true;
					scope.runRedirectActions();
				}, timeout);
			}
		}
	};

	angular
		.module('formFactory')
		.directive('ffCallback', ['$compile', '$log', '$window', '$timeout', input]);
})();ff.RenderingDirectives.FFFormController = function ($scope, $log, _, contextualData, $http, Upload, ffFormDataFactory, ffCommonUseFactory, ffDataFactory, i18n, $window) {
    // Injecting $scope just for comparison
    var vm = this;

    //Initialize watch to wait for the formController to be available.
    var deregisterFormControllerWatch = $scope.$watch(function() {
        return $scope[vm.formName];
    },  function(newValue) {
        if (!_.isUndefined(newValue)) {
            deregisterFormControllerWatch();
            //initialize watch for pristine status
            $scope.$watch(function() {
                return $scope[vm.formName].$pristine;
            }, function(newValue) {
                vm.formStepStatus[vm.currentStep].touched = !newValue;
                //only set the resetDisabled flag if we previously initialized it
                if (vm.currentForm.steps[vm.currentStep].resetDisabled !== null) {
                    vm.currentForm.steps[vm.currentStep].resetDisabled = newValue;
                }
            });
        }
    });
    activate();
    function activate() {
        vm.formId = $scope.vm.id.split('_')[0];
        return getFormData().then(function() {
            $log.info('FFFormController is activated');
            vm.activated = true;
        });
    }

    function getFormData() {
        return ffFormDataFactory.getFormData($scope.vm.locale, vm.formId, contextualData.isPreviewMode).then(function(data) {
            //Add reset management to form
            for (var i in data.steps) {
                if (i == 0) {
                    //initialize the first step
                    data.steps[i].resetDisabled = true;
                } else {
                    data.steps[i].resetDisabled = null;
                }
            }
            vm.currentForm = data;
            vm.originalForm = angular.copy(data);
            vm.displaySubmissionText = false;
            vm.runCallbacks = false;
            // Concatenate ff to avoid issue with string starting with numbers.
            vm.formName = 'ff' + data.formName.replace(/[\-]/g, '');
            vm.currentStep = 0;
            vm.results = {};
            vm.update = update;
            vm.manageFormState = manageFormState;
            vm.captureInputsState = captureInputsState;
            vm.setInputsStatus = setInputsStatus;
            //vm.goToStep = goToStep;
            vm.getFormController = getFormController;
            vm.reset = reset;
            vm.submit = submit;
            vm.preventSubmit = preventSubmit;
            vm.handleSubmission = handleSubmission;
            vm.handleRedirectIfPresent = handleRedirectIfPresent;
            vm.saveForm = saveForm;
            vm.containsFile = false;
            vm.isSubmitted = isSubmitted;
            vm.ffTestsInitMethod = ffTestsInitMethod;
            vm.displayInputs = displayInputs;
            vm.populateAndSubmitResults = populateAndSubmitResults;
            //vm.submitToMF = submitToMF;
            vm.uploads = {};
            vm.copyInputs = copyInputs;
            vm.getPreviousStepInputValue = getPreviousStepInputValue;
            vm.getCurrentStepInputValue = getCurrentStepInputValue;
            vm.switchForm = switchForm;
            //Resolvers
            vm.resolveLogic = resolveLogic;
            vm.resolvePlaceholder = resolvePlaceholder;
            vm.resolveInputsize = resolveInputsize;
            vm.resolveName = resolveName;
            vm.resolveLabel = resolveLabel;
            vm.parseDate = parseDate;
            vm.isPreviewMode = contextualData.isPreviewMode;
            vm.isLiveMode = isLiveMode;
            vm.getCaptchaKey = getCaptchaKey;
            vm.getUniqueId = getUniqueId;
            vm.captchaLoaded = false;
            vm.notifyOfCaptchaLoad = notifyOfCaptchaLoad;
            vm.setUpCallbackSystem = setUpCallbackSystem;
            ///Layout mode management
            if (!_.isUndefined(vm.currentForm.layoutJson) && !_.isEmpty(vm.currentForm.layoutJson)) {
                vm.currentForm.layoutJson = JSON.parse(vm.currentForm.layoutJson);
                vm.isLayoutAvailable = vm.currentForm.layoutJson.active;
            } else {
                vm.isLayoutAvailable = false;
            }
            vm.prepareInputsForLayout = prepareInputsForLayout;
            vm.reset();
            init();
            vm.setUpCallbackSystem();
        });
    }

    /**
     * Sends of an event listening parties notifying that callbacks can be registered.
     * Provides and easy to use mechanism to notify FF when callbacks are completed.
     */
    function setUpCallbackSystem() {
        //Notify interested parties that form is ready to accept callbacks
        angular.element(document).ready(function() {
            var formReadyEvent = new Event('ffFormReady');
            formReadyEvent.formInfo = {
                formName: vm.formName,
                notifyFormFactoryOfCompletion: function(callbackName, data) { //Called when callback is done.
                    if (callbackName === undefined || callbackName === "" || callbackName === null) {
                        console.error("No callbackName specified! You must specify a callback name.");
                        return;
                    }
                    var event = new Event('ff' + this.formName + 'NotifyComplete');
                    event.callbackResult = {};
                    event.callbackResult.callbackName = callbackName;
                    event.callbackResult.data = data;
                    document.dispatchEvent(event);
                },
                registerCallback: function(obj) { //Called when ffFormReady event is received
                    if (obj.fcn === undefined || !(typeof obj.fcn === 'function')) {
                        console.error("Improper callback object supplied. Read documentation!!!");
                    }
                    else {
                        if (!callbackRegistered(obj)) {
                            window.ffCallbacks[this.formName].push(obj);
                        }
                    }
                }
            };
            if ($window.ffCallbacks === undefined) {
                $window.ffCallbacks = {};
            }
            if ($window.ffCallbacks.registerCallback === undefined) {
                $window.ffCallbacks.registerCallback = function(formInfoObject, callbackRef, context) {
                    var obj = {
                        fcn:callbackRef,
                        ctx:context,
                        formInfo: formInfoObject
                    };
                    formInfoObject.registerCallback(obj);
                };
            }
            $window.ffCallbacks[vm.formName] = [];
            $window.document.dispatchEvent(formReadyEvent);
        });

        function callbackRegistered(obj) {
            for (var i in window.ffCallbacks[vm.formName]) {
                if (window.ffCallbacks[vm.formName][i].fcn === obj.fcn) {
                    return true;
                }
            }
            return false;
        }
    }

    function init() {
        vm.formStepStatus = {};
        vm.inputsStatus = {
            isDirty: {},
            ffVisible: {}
        };
        //initialize default status for all inputs
        //initialize the forms default status
        _.each(vm.currentForm.steps, function(step, index) {
            vm.formStepStatus[index] = {touched: false};
            _.each(step.inputs, function(input) {
                vm.inputsStatus.isDirty[input.name] = false;
                vm.inputsStatus.ffVisible[input.name] = false;
            });
        });
    }

    function notifyOfCaptchaLoad() {
        vm.captchaLoaded = true;
    }

    /**
     * Retrieves the captcha key if it exists
     * @function
     * @returns {*}
     */
    function getCaptchaKey() {
        if (contextualData.captchaApiKey !== undefined && contextualData.captchaApiKey !== '') {
            return contextualData.captchaApiKey;
        } else {
            return null;
        }
    }

    /**
     * Creates a unique ID
     * @function
     * @param length - the length generated ID should be
     * @returns {string} - generated ID
     */
    function getUniqueId(length) {
        var chars = 'ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz'.split('');

        if (!length) {
            length = Math.floor(Math.random() * chars.length);
        }

        var str = '';
        for (var i = 0; i < length; i++) {
            str += chars[Math.floor(Math.random() * chars.length)];
        }
        return str;
    }

    function switchForm() {
        ff.dataProvider = vm.currentTestFile.filePath;
        getFormData();
        $scope.$watch(function() {
            return !_.isUndefined($scope[vm.formName]);
        },function(newValue) {
            if (newValue) {
                vm.reset();
            }
        });
    }

    function collectTestData() {
        $http.get(contextualData.context + '/modules/formfactory/live/jsonformfiles').then(function (data) {
            vm.testFiles = data.data.files;
            vm.currentTestFile = null;
        });
    }

    /**
     * Submit the form if it hasn't already been submitted
     * @function
     */
    function submit() {
        var form = $scope[vm.formName];
        if (form.$valid) {
            if (form.$submitted) {
                $log.info('form already submitted');
            } else {
                vm.handleSubmission();
                form.$setSubmitted(true);
            }
        }
    }

    /**
     * Prevent default behaviour of submitting.
     * @function
     * @param e - event obj
     */
    function preventSubmit(e) {
        e.preventDefault();
        $scope[vm.formName].$submitted = false;
    }

    /**
     * Resets the form to it's default state
     * @function
     */
    function reset() {
        if (!_.isUndefined($scope[vm.formName])) {
            $scope[vm.formName].$setPristine();
            $scope[vm.formName].$setUntouched();
        }
        vm.inputs = angular.copy(vm.originalForm.steps[vm.currentStep].inputs);
        if (vm.isLayoutAvailable) {
            //Reinitialize rows to update rendering.
            vm.currentForm.layoutJson.steps[vm.currentStep].rows = angular.copy(vm.currentForm.layoutJson.steps[vm.currentStep].rows);
            vm.prepareInputsForLayout();
        }
    }

    /**
     * This function is responsible for updating the form as we back and forth through the steps
     * or attempt to submit the form.
     * @function
     * @param advance - boolean, whether we are advancing to the next step
     */
    function update(advance) {
        var form = $scope[vm.formName];
        vm.copyInputs();
        //capture current step's inputs before moving forward or backwards
        if (vm.currentStep < vm.currentForm.steps.length) {
            vm.captureInputsState();
        }
        if (advance && form.$valid) {
            vm.currentStep++;
        } else if (!advance) {
            if (vm.currentStep !== 0) {
                vm.currentStep--;
            }
        }
        //Once we have moved forward/backward set up current step's inputs status
        if (vm.currentStep < vm.currentForm.steps.length) {
            vm.setInputsStatus();
        }
        vm.inputs = [];
        if (vm.currentStep < vm.currentForm.steps.length) {
            vm.manageFormState();
            vm.displayInputs();
        } else if (!vm.isPreviewMode) {
            vm.populateAndSubmitResults();
        }
    }

    /**
     * Copies the values of the inputs on the current step of the form into the form obj.
     * @function
     */
    function copyInputs() {
        vm.currentForm.steps[vm.currentStep].inputs = [];
        _.forEach(vm.inputs, function(input) {
            if (input.value instanceof FileList) {
                vm.containsFile = true;
                var fileList = input.value;
                input.value = null;
                var cloned = _.clone(input);
                cloned.value = fileList;
                vm.currentForm.steps[vm.currentStep].inputs.push(cloned);
            } else {
                vm.currentForm.steps[vm.currentStep].inputs.push(_.clone(input));
            }
        });
    }

    /**
     * Copies existing values from the form obj for the current step into the corresponding inputs.
     * @function
     */
    function displayInputs() {
        //Display the current steps' results
        _.forEach(vm.currentForm.steps[vm.currentStep].inputs, function(input) {
            if (input.value instanceof FileList) {
                var fileList = input.value;
                input.value = null;
                var cloned = _.clone(input);
                cloned.value = fileList;
                vm.inputs.push(cloned);
            } else {
                vm.inputs.push(_.clone(input));
            }
        });
        if (vm.isLayoutAvailable) {
            vm.prepareInputsForLayout();
        }
    }

    /**
     * Gathers all the available values for the inputs to be submitted
     * @function
     */
    function populateAndSubmitResults() {
        //Generate Results
        _.forEach(vm.currentForm.steps, function(step) {
            _.forEach(step.inputs, function(input) {
                if (!_.isUndefined(input.value) && (!_.isEmpty(input.value) || _.isNumber(input.value) || _.isBoolean(input.value) || moment(input.value).isValid())) {
                    vm.results[input.name] = input.value;
                }
            });
        });
        vm.results["token"] = vm.currentForm.token;
        vm.submit();
    }

    /**
     * Handles form submission by first collecting and uploading files in temp folder if any
     * and then calling vm.saveForm().
     *
     * @function
     * @memberOf FFFormController
     */
    function handleSubmission() {
        var fileCounter = 0;
        var fileUploadResponses = {};
        var fileLists = {};
        if (vm.containsFile) {
            //We want to prevent execution of uploads until we determine the number of files.
            _.each(vm.results, function(value, key) {
                if (value instanceof FileList) {
                    fileCounter++;
                    fileLists[key] = value;
                }
            });

            //Upload files and then save form if upload was successful
            _.each(fileLists, function(list, key) {
                //If no mime type detected, default to application/octet-stream
                var fileType = !_.isEmpty(list[0].type) ? encodeURI(list[0].type.replace('/', '_')) : 'application_octet-stream';
                Upload.http({
                    url: contextualData.context + '/modules/formfactory/live/fileupload/' + vm.formId + '/' + key + '/' + encodeURI(list[0].name + '/' + fileType),
                    headers: {
                        'Content-Type': 'application/octet-stream'
                    },
                    data: list[0]
                }).then(function(result) {
                    fileCounter--;
                    fileUploadResponses[key] = [result.data.data.tempFileName,
                        result.data.data.originalFileName,
                        result.data.data.type];
                    if (fileCounter === 0) {
                        //vm.submitToMF();
                        vm.saveForm(fileUploadResponses);
                    }
                }, function(result) {
                    $log.error(result);
                }, function(progress) {
                    vm.uploads[progress.config.data.name] = (progress.loaded / progress.total) * 100 + '%';
                });
            });
        } else { //Save form
            //vm.submitToMF();
            vm.saveForm();
        }
    }

    /**
     * Calls 'save' API
     * Note that uploadResponses has the following structure :
     * {"ffUploadedFile_0": [ "/Users/.../ffFileUpload7093304331231541845.tmp", "originalName.jpg", "fileupload_0_1", "image/jpeg" ], "ffUploadFileNumber": 1 } }
     *
     * @function
     * @memberOf FFFormController
     * @param {array} uploadResponses
     */
    function saveForm(uploadResponses) {
        if (uploadResponses) {
            _.each(uploadResponses, function(key, value) {
                vm.results[value] = key;
            });
        }

        var req = {
            method: 'POST',
            url: contextualData.context + '/modules/formfactory/live/' + vm.formId + '/' + $scope.vm.locale,
            data: vm.results
        };

        $http(req)
            .success(function (data) {
                setSubmissionText('ff.message.submission');
                //Will show custom callback directive
                buildCallbackObjectIfNeedBe();

                if (vm.currentForm.callbacks !== null) {
                    vm.actionData = data;
                    vm.resultData = getResultData();
                    vm.runCallbacks= true;
                    return;
                }
                if (!handleRedirectIfPresent(data)) {
                    vm.displaySubmissionText = true;
                }
                $log.info('Form successfully saved!!!!!');
            })
            .error(function(error) {
                vm.currentForm.afterSubmissionText = i18n.message('ff.message.submissionError');
                vm.displaySubmissionText = true;
                $log.info('form was not submitted due to ERROR!!!!');
            });

        function setSubmissionText(label) {
            if (vm.currentForm.afterSubmissionText === null || vm.currentForm.afterSubmissionText === '') {
                vm.currentForm.afterSubmissionText = i18n.message(label);
            }
        }
    }

    /**
     * Returns the results of the submitted form
     * @function
     * @returns {{}} object containing results
     */
    function getResultData() {
        var data = {};
        _.each(vm.results, function(value, key) {
            if (key !== 'token') {
                data[key] = value;
            }
        });
        return data;
    }

    /**
     * Creates the callbacks object if there are callbacks attached to this form but the container is null.
     *@function
     */
    function buildCallbackObjectIfNeedBe() {
        if (vm.currentForm.callbacks === null && $window.ffCallbacks[vm.formName].length > 0) {
            vm.currentForm.callbacks = {
                callbacks: [],
                displayTemplates: false
            }
        }
    }

    /**
     * Processes redirects if they are present (will redirect to url on the last found redirect)
     *
     * @param data
     * @returns {boolean}
     */
    function handleRedirectIfPresent(data) {
        var redirectAction = {};
        redirectAction.name = '';
        //Find last redirect action
        for (var i in data) {
            if ("actionName" in data[i] && (data[i].actionName[0] === 'redirectToAPage' || data[i].actionName[0] === 'redirectToUrl')) {
                redirectAction.url = data[i].redirectUrl[0];
                redirectAction.name = data[i].actionName[0];
            }
        }
        //Perform redirect
        if (redirectAction.name === 'redirectToAPage') {
            window.location.assign(redirectAction.url);
            return true;
        }
        if (redirectAction.name === 'redirectToUrl') {
            window.location.assign(redirectAction.url);
            return true;
        }
        return false;
    }

    /**
     * Gets the form controller
     * @function
     * @returns {*} form controller obj
     */
    function getFormController() {
        return $scope[vm.formName];
    }

    function isSubmitted() {
        return vm.currentStep == vm.currentForm.steps.length;
    }

    /**
     * Manage the form state based on whether the current step has been previously filled out
     * @function
     */
    function manageFormState() {
        //if we have not been on this step before the resetDisabled will be initialized as null
        if (vm.currentForm.steps[vm.currentStep].resetDisabled === null) {
            //set form status to match our current state
            vm.getFormController().$setPristine();
            vm.currentForm.steps[vm.currentStep].resetDisabled = true;
        } else {
            //set the form to the appropriate state based on what we previously did on this step.
            if (vm.currentForm.steps[vm.currentStep].resetDisabled) {
                vm.getFormController().$setPristine();
            } else {
                vm.getFormController().$setDirty();
            }
        }
    }

    /**
     * Call this function with javascript like so:
     * angular.element(document.getElementById("'c1917f57-6c88-43b8-b7b4-5e9d57caa28e'")).scope().$$childHead.vm.ffTestsInitMethod('/modules/form-factory-core/javascript/angular/components/formrendering/testdata/allFormTestData.json');
     * replace url and form id to match current page.
     *
     * @param url
     * @returns {boolean}
     */
    function ffTestsInitMethod(obj) {
        vm.currentForm = obj;
        vm.originalForm = angular.copy(obj);
        vm.formName = obj.formName;
        vm.reset();
        $scope.$digest();
        return true;
    }

    /**
     * Will search the previous steps for the specified input and return its value if found.
     * @function
     * @memberOf FFFormController
     * @param inputId id of the input we are looking for
     * @returns {*}
     */
    function getPreviousStepInputValue(inputId) {
        var searchStep = vm.currentStep - 1;
        if (vm.currentStep > 0 && searchStep >= 0) {
            for (searchStep; searchStep >= 0; searchStep--) {
                var searchedInput = _.find(vm.currentForm.steps[searchStep].inputs, function(input) {
                    if (input.name === inputId) {
                        return true;
                    }
                });
                if (!_.isEmpty(searchedInput)) {
                    return {value: searchedInput.value, validity: null};
                }

            }
        }
        return undefined;
    }

    /**
     * Searches for a value of a field (inputId) in the input of current step.
     *
     * @function
     * @memberOf FFFormController
     * @param inputId
     * @returns {*}
     */
    function getCurrentStepInputValue(inputId) {
        var formController = vm.getFormController();
        var searchedInput = _.find(vm.inputs, function(input) {
            if (input.name === inputId) {
                return true;
            }
        });
        if (!_.isEmpty(searchedInput)) {
            return {value: searchedInput.value, validity: formController[searchedInput.name].$valid};
        }
        return null;
    }

    /**
     * This function captures the inputs current dirty and visible status
     * to be used when/if returning to the current step
     * @function
     * @memberOf FFFormController
     */
    function captureInputsState() {
        var formController = vm.getFormController();
        _.each(vm.currentForm.steps[vm.currentStep].inputs, function(input) {
            if (formController[input.name] !== undefined) {
                vm.inputsStatus.isDirty[input.name] = formController[input.name].$dirty;
                vm.inputsStatus.ffVisible[input.name] = formController[input.name].ffVisible;
            }
        });
    }

    /**
     * This function will gather all the inputs on the current step and :
     * Mark the input as dirty if it was previously filled
     * Set its last known visibly status.
     * @function
     * @memberOf FFFormController
     */
    function setInputsStatus() {
        var formController = vm.getFormController();
        _.each(vm.currentForm.steps[vm.currentStep].inputs, function(input) {
            if (vm.inputsStatus.isDirty[input.name]) {
                //verify that input controller has been attached to the form controller
                //otherwise we add a watch and wait for it to be attached
                if (_.isUndefined(formController[input.name])) {
                    var deregisterInputWatch = $scope.$watch(
                        //once the input controller has been attached to the form
                        //we set the inputs dirty status
                        //and update its visibility
                        function() {
                            return !_.isUndefined(formController[input.name]);
                        },
                        function(newValue) {
                            if (newValue) {
                                deregisterInputWatch();
                                formController[input.name].$setDirty();
                                formController[input.name].ffVisible = vm.inputsStatus.ffVisible[input.name];
                            }
                        }
                    );
                } else {
                    //if the controller has already been attached, then we can safely set its status
                    formController[input.name].$setDirty();
                    formController[input.name].ffVisible = vm.inputsStatus.ffVisible[input.name];
                }
            }
        });
    }

    function resolveLogic(input, form) {
        return ffCommonUseFactory.logicResolver(input, form);
    }

    function resolvePlaceholder(input) {
        return ffCommonUseFactory.placeholderResolver(input);
    }

    function resolveInputsize(input) {
        return ffCommonUseFactory.inputsizeResolver(input);
    }

    function resolveName(input) {
        return ffCommonUseFactory.nameResolver(input);
    }

    function resolveLabel(input) {
        return ffCommonUseFactory.labelResolver(input);
    }

    function parseDate(value) {
        return ffCommonUseFactory.parseDate(value);
    }

    function isLiveMode() {
        return true;
    }

    /**
     * Prepares the inputs to be easily displayed when using a layout.
     * @function
     */
    function prepareInputsForLayout() {
        vm.preparedLayoutInputs = {};
        _.each(vm.inputs, function(input, index) {
            vm.preparedLayoutInputs[input.layoutUUID] = index;
        });
    }
};

ff.RenderingDirectives.FFFormController.$inject = ['$scope', '$log', '_', 'contextualData', '$http', 'Upload', 'ffFormDataFactory', 'ffCommonUseFactory', 'ffDataFactory', 'i18nService', '$window'];
(function () {
    'use strict';

    ff.Factories.formDataFactory = function ($http, $log, contextualData) {
        return {
            getFormData: getFormData
        };

        function getFormData(locale, formId, inPreviewMode) {
            if (ff.dataProvider === null) {
                if (inPreviewMode !== "") {
                    return $http.get(contextualData.context + '/modules/formfactory/live/preview/' + formId + '/' + locale).then(getFormDataComplete).catch(getFormDataFailed);
                }
                else {
                    return $http.get(contextualData.context + '/modules/formfactory/live/' + formId + '/' + locale).then(getFormDataComplete).catch(getFormDataFailed);
                }
            }

            function getFormDataComplete(response) {
                $log.debug('ffFormDataFactory completed data transfer!');
                $log.debug(response);
                return response.data;
            }

            function getFormDataFailed(error) {
                $log.debug('XHR Failed for getFormData.' + error.data);
            }
        }
    };

    ff.Factories.formDataFactory.$inject = ['$http', '$log', 'contextualData'];

    angular
        .module('formFactory')
        .factory('ffFormDataFactory', ff.Factories.formDataFactory);
})();(function () {
    'use strict';
    ff.RenderingDirectives.renderInput = function ($compile, $log) {
        var directive = {
            restrict: 'E',
            require : ['^ffController'],
            scope   : {
                input             : '=',
                resolveLogic      : '&',
                resolvePlaceholder: '&',
                resolveInputsize  : '&',
                resolveName       : '&',
                resolveLabel      : '&'
            },
            link    : postLinkFunc
        };

        return directive;

        function postLinkFunc(scope, el, attr, ffControllerCtrl) {
            var formController = ffControllerCtrl[0];
            scope.form = formController.getFormController();
            scope.readOnly = !formController.isLiveMode();
            el.append($compile(angular.element(scope.input.template).attr('view-type', !_.isUndefined(formController.currentForm.theme) && formController.currentForm.theme !== null ? formController.currentForm.theme : ''))(scope));

            /**
             * Returns whether the input is required
             * @function
             * @returns {boolean|*}
             */
            scope.isRequired = function isRequired() {
                var inputModel = scope.form[scope.input.name];
                var required   = _.isUndefined(scope.input.validations) ? false : (!_.isUndefined(scope.input.validations.required) || !_.isUndefined(scope.input.validations['matrix-required']));
                return (required && _.isUndefined(inputModel)) || (required && inputModel.ffVisible);
            };

            /**
             * Determines if the checkbox input is required
             * @function
             * @returns {*}
             */
            scope.checkValues = function checkValues() {
                var model = scope.form[scope.input.name];
                var required   = _.isUndefined(scope.input.validations) ? false : !_.isUndefined(scope.input.validations.required);

                if (required && _.isUndefined(model)) {
                    return _.find(scope.input.value,function(value) {
                            return value;
                        }) == undefined;
                } else if (required && model.ffVisible) {
                    return _.find(scope.input.value,function(value) {
                            return value;
                        }) == undefined && model.ffVisible;
                }
            };

            /**
             * Changes status of input to dirty
             * @function
             */
            scope.makeDirty = function makeDirty() {
                var model = scope.form[scope.input.name];
                model.$dirty = true;
            }
        }
    };

    angular
        .module('formFactory')
        .directive('ffInput', ['$compile', '$log', ff.RenderingDirectives.renderInput]);
})();
(function() {
    'use strict';

    ff.LogicDirectives.applyLogicRulesInput = function ($log, _, LogicService, $timeout) {
        var directive = {
            restrict: 'A',
            require : ['^ffController', '?ngModel'],
            scope   : false,
            link    : linkFunc
        };

        return directive;

        function linkFunc(scope, el, attr, ctrls) {
            var model = ctrls[1];
            var input = scope.input;

            if (!_.isUndefined(model) && model !== null) {
                var listener    = function (newValue, oldValue, scope) {
                    //Get value and name of the edited field
                    var val = newValue;
                    newValue = val.value;
                    var fieldName = val.fieldName;

                    $timeout(function() {
                        //If new value is undefined then field is not visible
                        if (_.isUndefined(newValue)) {
                            model.ffVisible = false;
                            return;
                        }

                        //If value has changed evaluate logics again
                        var isVisible = true;
                        _.each(model.$logics, function (logicRule, ruleFieldReference) {
                            //Execute logic rule for edited field only and remember value
                            if (ruleFieldReference.split(':')[1] === fieldName) {
                                if (_.isFunction(logicRule)) {
                                    LogicService.$logicRulesValues[model.$name][ruleFieldReference] = logicRule(scope, newValue);
                                } else {
                                    $log.error("Trying to call a logic rule that is not a function");
                                    $log.error(logicRule);
                                    isVisible = false;
                                }
                            }
                        });

                        //Check aggregate value of logic rules
                        var rulesAggregateValue = true;
                        if (isVisible) {
                            _.each(LogicService.$logicRulesValues[model.$name], function(value) {
                                rulesAggregateValue = rulesAggregateValue && value;
                            });
                            isVisible = rulesAggregateValue;
                        }
                        //Update the visibility status
                        model.ffVisible = isVisible;
                        if (model.ffVisible) {
                            //Run the validations, needs to run in a timeout so it is not reset but the visibility toggle timeout.
                            $timeout(function() {
                                model.$validate();
                            });
                        }
                    });
                };
                //Initialize ffVisible to true if there is no logics, otherwise false
                model.ffVisible = _.isUndefined(input.logics);
                model.$logics   = {};
                watchVisibilityToggle(scope, input);

                angular.forEach(scope.input.logics, function (rule) {
                    // Check if the source of the rule is already available in the controller or is coming from a previous step
                    // otherwise watch on the controller
                    var ruleName = getRuleName(rule.name);
                    var formController     = ctrls[0].getFormController();
                    var searchedInputValue = ctrls[0].getPreviousStepInputValue(rule.fieldName);
                    if (_.isUndefined(searchedInputValue) && _.isUndefined(formController[rule.fieldName])) {
                        var input           = scope.input;
                        var deregisterWatch = scope.$watch(function() {
                            //wait for the controller of this field to be generated
                            return _.isUndefined(formController[rule.fieldName]);
                        }, function() {
                            if (!_.isUndefined(formController[rule.fieldName])) {
                                angular.forEach(input.logics, function (rule) {
                                    var ruleName = getRuleName(rule.name);
                                    ctrls[1] = model;
                                    LogicService.initLogicRule(attr.$normalize(ruleName), rule, scope, el, attr, ctrls);
                                });
                                watchRuleFieldValueToApplyLogic(formController, rule, listener);
                                // Rules have been initialized so we remove the watch
                                deregisterWatch();
                            }
                        });
                    } else {
                        LogicService.initLogicRule(attr.$normalize(ruleName), rule, scope, el, attr, ctrls);
                        watchRuleFieldValueToApplyLogic(formController, rule, listener);
                    }
                });
                //Set initial value of each logic rule for each source input, keep track of values
                //these values will be used to determine overall validity. Initially every rule value is false
                if (_.isUndefined(LogicService.$logicRulesValues[model.$name])) {
                    LogicService.$logicRulesValues[model.$name] = {};
                    _.each(model.$logics, function (logicRule, ruleFieldReference) {
                        LogicService.$logicRulesValues[model.$name][ruleFieldReference] = false;
                    });
                }
            } else {
                $log.debug("There is no model for this input "+scope.input.name);
                var formController  = ctrls[0].getFormController();
                var deregisterWatch = scope.$watch(function () {
                    return _.isUndefined(formController[input.name]);
                }, function () {
                    if (!_.isUndefined(formController[input.name])) {
                        var inputModelController       = formController[input.name];
                        inputModelController.ffVisible = _.isUndefined(input.logics);
                        inputModelController.$logics   = {};

                        watchVisibilityToggle(scope, input);
                        var listener    = function (newValue, oldValue, scope) {
                            //Get value and name of the edited field
                            var val = newValue;
                            newValue = val.value;
                            var fieldName = val.fieldName;

                            $timeout(function() {
                                if(_.isUndefined(newValue)){
                                    inputModelController.ffVisible = false;
                                    return;
                                }

                                //If value has changed evaluate logics again
                                var isVisible = true;
                                _.each(inputModelController.$logics, function (logicRule, ruleFieldReference) {
                                    //Execute logic rule for edited field only and remember value
                                    if (ruleFieldReference.split(':')[1] === fieldName) {
                                        if (_.isFunction(logicRule)) {
                                            LogicService.$logicRulesValues[inputModelController.$name][ruleFieldReference] = logicRule(scope, newValue);
                                        } else {
                                            $log.error("Trying to call a logic rule that is not a function");
                                            $log.error(logicRule);
                                            isVisible = false;
                                        }
                                    }
                                });

                                //Check aggregate value of logic rules
                                var rulesAggregateValue = true;
                                if (isVisible) {
                                    _.each(LogicService.$logicRulesValues[inputModelController.$name], function(value) {
                                        rulesAggregateValue = rulesAggregateValue && value;
                                    });
                                    isVisible = rulesAggregateValue;
                                }
                                inputModelController.ffVisible = isVisible;
                                if (inputModelController.ffVisible) {
                                    //Run the validations, needs to run in a timeout so it is not reset but the visibility toggle timeout.
                                    $timeout(function() {
                                        inputModelController.$validate();
                                    });
                                }
                            });
                        };
                        angular.forEach(input.logics, function (rule) {
                            var ruleName = getRuleName(rule.name);
                            ctrls[1] = inputModelController;
                            LogicService.initLogicRule(attr.$normalize(ruleName), rule, scope, el, attr, ctrls);
                            watchRuleFieldValueToApplyLogic(formController, rule, listener);
                        });
                        deregisterWatch();

                        //Set initial value of each logic rule for each source input, keep track of values
                        //these values will be used to determine overall validity. Initially every rule value is false
                        LogicService.$logicRulesValues[inputModelController.$name] = {};
                        _.each(inputModelController.$logics, function (logicRule, ruleFieldReference) {
                            LogicService.$logicRulesValues[inputModelController.$name][ruleFieldReference] = false;
                        });
                    }
                });
            }


            // This function will empty any non visible field so they are not part of the submission
            function watchVisibilityToggle(scope, input) {
                scope.$watch(function (scope) {
                    return scope.form[input.name].ffVisible;
                }, function (newValue, oldValue) {
                    if (!_.isUndefined(newValue) && !newValue && scope.form[input.name].$dirty) {
                        if (_.isArray(input.value)) {
                            input.value = [];
                        } else if (_.isObject(input.value)) {
                            input.value = {};
                        } else {
                            input.value = null;
                        }
                    }
                    if (!_.isUndefined(newValue) && !newValue) {
                        //Reset the validations
                        $timeout(function() {
                            //This is done in a timeout so that we execute the reset after the validations have run.
                            _.each(scope.form[input.name].$validators, function(validation, validationName) {
                                scope.form[input.name].$setValidity(validationName, true);
                            });
                        });
                    }
                });
            }

            /**
             * This function finds source input at attaches a watch on its value or takes the value from
             * previous step if there is no match in the current step.
             *
             * @param formController
             * @param rule
             * @param listener
             */
            function watchRuleFieldValueToApplyLogic(formController, rule, listener) {
                if (!_.isUndefined(formController[rule.fieldName])) {
                    registerValueWatches(ctrls, rule, listener);
                } else {
                    var result = ctrls[0].getPreviousStepInputValue(rule.fieldName);
                    if (_.isUndefined(result)) {
                        result = {};
                        result.value = undefined;
                        result.validity = undefined;
                    }
                    var previousStepInputValue = { value:result.value, validity: result.validity, fieldName: rule.fieldName};
                    if (!_.isUndefined(previousStepInputValue.value)) {
                        //console.log("field value for " + rule.fieldName + " found in previous step:" + previousStepInputValue);
                        listener(previousStepInputValue);
                    }
                    else {
                        //The value is not in the previous step and we have to wait for the input to be fully loaded in the form controller
                        var unregisterFieldNameWatch = scope.$watch(function() {
                            return !_.isUndefined(formController[rule.fieldName]);
                        }, function(newValue) {
                            if (newValue) {
                                registerValueWatches(ctrls, rule, listener);
                                unregisterFieldNameWatch();
                            }
                        });

                    }
                }
            }

            /**
             * Here we register watches to watch for value changes in the source field.
             *
             * @param ctrls
             * @param rule
             * @param listener
             */
            function registerValueWatches(ctrls, rule, listener) {
                var inputValue = ctrls[0].getCurrentStepInputValue(rule.fieldName);
                if(_.isUndefined(inputValue) || _.isNull(inputValue)) {
                    scope.$watch(function (scope) {
                        var result = ctrls[0].getCurrentStepInputValue(rule.fieldName);
                        return { value:result.value, validity: result.validity, fieldName: rule.fieldName};
                    }, listener, true);
                } else {
                    scope.$watch(function (scope) {
                        var result = ctrls[0].getCurrentStepInputValue(rule.fieldName);
                        return { value:result.value, validity: result.validity, fieldName: rule.fieldName};
                    }, listener, true);

                }
            }

            function getRuleName(ruleType) {
                return ruleType.split('_')[0];
            }
        }
    };

    angular.module('formFactory')
        .directive('ffLogic', ['$log', '_', 'LogicService', '$timeout', ff.LogicDirectives.applyLogicRulesInput]);
})();(function() {
    'use strict';
    ff.RenderingDirectives.parseJSON = function($log, ValidationService) {
        var directive = {
            restrict: 'A',
            require: ['^ffController', '?ngModel'],
            scope: false,
            link: postLinkFunc
        };
        return directive;

        function postLinkFunc(scope, el, attr, ctrls) {
            var model  = ctrls[1];
            var parser = function(inputValue) {
                if (_.isString(inputValue)) {
                    var transformedInput = JSON.parse(inputValue);
                    model.$setViewValue(transformedInput);
                    return transformedInput;
                }
                return inputValue;
            };
            if (!_.isUndefined(model) && model != null) {
                model.$parsers.push(parser);
            } else {
                var formController  = ctrls[0].getFormController();
                var input           = scope.input;
                var deregisterWatch = scope.$watchCollection(formController, function() {
                    if (!_.isUndefined(formController[input.name])) {
                        var model = formController[input.name];
                        model.$parsers.push(parser);
                        deregisterWatch();
                    }
                });
            }

        }
    };

    angular.module('formFactory')
        .directive('ffParseJson', ['$log', ff.RenderingDirectives.parseJSON]);
})();
(function() {
    'use strict';

    ff.ValidationDirectives.validateInput = function ($log, ValidationService) {
        var directive = {
            restrict: 'A',
            require : ['^ffController', '?ngModel'],
            scope   : false,
            link    : postLinkFunc
        };
        return directive;

        function postLinkFunc(scope, el, attr, ctrls) {
            var model = ctrls[1];
            var input = scope.input;
            //Register validations if model is available otherwise setup a watch and wait
            //for the model to be initialized and then register the validations.
            if (!_.isUndefined(model) && model != null) {
                angular.forEach(scope.input.validations, function (rule, ruleName) {
                    if (ruleName != 'required') {
                        ValidationService.initValidator(attr.$normalize(ruleName), rule, scope, el, attr, ctrls);
                    }
                });
            } else {
                var formController  = ctrls[0].getFormController();
                var deregisterWatch = scope.$watchCollection(formController, function () {
                    if (!_.isUndefined(formController[input.name])) {
                        angular.forEach(scope.input.validations, function (rule, ruleName) {console.log('rule: ' + ruleName)
                            if (ruleName != 'required') {
                                ctrls[1] = formController[input.name];
                                ValidationService.initValidator(attr.$normalize(ruleName), rule, scope, el, attr, ctrls);
                            }
                        });
                        deregisterWatch();
                    }
                });

            }
            /**
             * Runs validations
             * @function
             */
            scope.ffValidate = function () {
                scope.form[input.name].$validate();
            };
        }
    };

    angular.module('formFactory')
        .directive('ffValidations', ['$log', 'ValidationService', ff.ValidationDirectives.validateInput]);
})();
(function() {
    'use strict';
    /**
     * @function
     * @name registerLogicRules
     * @param {array} logicRules
     * @memberOf ff
     */
    ff.registerLogicRules = function(logicRules) {
        angular.module('formFactory').config(['LogicServiceProvider', function(LogicServiceProvider) {
            angular.forEach(logicRules, function(value, key) {
                LogicServiceProvider.registerLogicRule(key, value);
            });
        }]);
    };

    /**
     * @class
     * @private
     * @memberOf ff
     */
    ff._LogicServiceProvider = function() {
        var logicRules = {};

        /**
         * @function
         * @param {string} name
         * @param {Function} fct
         */
        this.registerLogicRule = function(name, fct) {
            logicRules[name] = fct;
        };

        /**
         * @function
         * @type {ff._LogicService}
         */
        this.$get = [function() {
            return new ff._LogicService(logicRules);
        }];
    };

    /**
     * @class
     * @param {Object[]} logicRules
     * @private
     * @memberOf ff
     */

    ff._LogicService = function(logicRules) {

        this.$logicRulesValues = {};
        /**
         * @function
         * @param {string} name
         * @param {Object} rule
         * @param {Object} scope
         * @param {Object} el
         * @param {string[]} attr
         * @param {Object[]} ctrls
         */
        this.initLogicRule = function(name, rule, scope, el, attr, ctrls) {
            if (_.isFunction(logicRules[name])) {
                ctrls[1].$logics[name + ':' + rule.fieldName] = logicRules[name](rule, scope, el, attr, ctrls);
            }
        };
    };

    angular.module('formFactory.logic', [])
        .provider('LogicService', ff._LogicServiceProvider);

    ff.registerLogicRules(ff.logicRules);
})();
(function(){
'use strict';
/**
 * Register a validation rules in the ValidationService Provider
 *
 * @function
 * @name registerValidators
 * @param {array} validationRules
 * @memberOf ff
 */
ff.registerValidators = function(validationRules) {
    angular.module('formFactory').config(['ValidationServiceProvider', function(ValidationServiceProvider) {
        angular.forEach(validationRules, function(value, key) {
            ValidationServiceProvider.registerValidator(key, value);
        });
    }]);
};

/**
 * Provider for ValidationService
 *
 * @class
 * @memberOf ff
 */
ff._ValidationServiceProvider = function() {
    var validators = {};

    /**
     * @function
     * @param {string} name
     * @param {function} fct
     */
    this.registerValidator = function(name, fct) {
        validators[name] = fct;
    };

    /**
     * @function
     * @type {ff._ValidationService}
     */
    this.$get = [function() {
        return new ff._ValidationService(validators);
    }];
};

/**
 * @class
 * @param {Object[]} validators
 * @memberOf ff
 */
ff._ValidationService = function(validators) {
    /**
     * @function
     * @param {string} name name
     * @param {Object} rule rule
     * @param {Object} scope scope
     * @param {Object} el el
     * @param {string[]} attr attr
     * @param {object[]} ctrls ctrls
     */
    this.initValidator = function(name, rule, scope, el, attr, ctrls, $parse) {
        if (validators[name] && _.isUndefined(ctrls[1].$validators[name])) {
            var validationRule = validators[name](rule, scope, el, attr, ctrls, $parse);
            if(_.isFunction(validationRule)) {
                ctrls[1].$validators[name] = validationRule;
            }
        }
    };
};

angular.module('formFactory.validation', [])
    .provider('ValidationService', ff._ValidationServiceProvider);

ff.registerValidators(ff.validationRules);
})();
