angular.module('formFactory')
       .directive('ffTypeahead', ['$log', ffTypeahead])
       .directive('ffTypeaheadHoover', ['$log', ffTypeaheadHoover]);

function ffTypeahead($log) {
    var directive = {
        restrict: 'E',
        require: ['^ffController'],
        templateUrl: '/modules/form-factory-core/javascript/angular/templates/typeaheadDefinition.html',
        link: linkFunction,
        controller: FFTypeaheadController,
        controllerAs: 'th',
        bindToController: true
    };

    return directive;

    function linkFunction(scope, el, attr, ctrls) {
        scope.th.setUpTH(el);
    }
}

FFTypeaheadController.$inject = ['$scope', '$log', 'ffTypeaheadFactory', '_', '$timeout'];

function FFTypeaheadController($scope, $log, ffTypeaheadFactory, _, $timeout) {
    var th = this;
    //console.log('Typeahead scope');
    //console.info($scope);
    th.emptyMessage             = 'No results found';
    th.showing                  = false;
    th.lastData                 = {};
    th.currentSelection         = {};
    th.url                      = $scope.input.url;
    th.paramName                = $scope.paramName;
    th.setUpTH                  = setUpTH;
    th.processRequest           = processRequest;
    th.showResults              = showResults;
    th.selectResult             = selectResult;
    th.hideTH                   = hideTH;
    th.showSuggestion           = showSuggestion;
    th.inputExists              = inputExists;

    loadTemplate();

    function loadTemplate() {
        if ($scope.input.useSuggestionTemplate) {
            return setUpTemplate().then(function() {
                $log.debug('ffTypeahead set up template');
            });
        }
        return;
    }

    function setUpTemplate() {
        return ffTypeaheadFactory.getTHData($scope.input.suggestionTemplate).then(function(data) {
            th.useSuggestionTemplae     = true;
            th.suggestionTemplate       = _.template(data);
        });
    }

    //Transforms DOM elements to set up display
    function setUpTH(el) {
        var inputWidth = el.find('input').width();
        var dropDown = el.find('.ffTypeaheadDropdown');
        dropDown.css({'position': 'absolute',
                        'background-color': 'white',
                        'width': inputWidth,
                        'margin-top': '5px',
                        'padding-top': '5px',
                        'padding-bottom': '5px',
                        'border': '1px solid #cccccc',
                        'border-radius': '10px',
                        'max-height': '200px',
                        'overflow-y': 'auto',
                        'overflow-x': 'none',
                        'z-index': '1032',
                        'box-shadow': '3px 3px 8px -3px rgba(0,0,0,1)'});
    }

    function processRequest() {
        var value = $scope.form[$scope.input.name].$modelValue;
        var urlToUse = th.url + '?' + th.paramName + '=' + value;

        ffTypeaheadFactory.getTHData(th.url).then(function(data) {
            //$log.info('Request processed');
            //$log.info(data);
            th.lastData = data;
            th.showResults();
        });
    }

    function showResults() {
        th.showing = true;
    }

    function selectResult(obj) {
        th.currentSelection = obj;
        $scope.form[$scope.input.name].$setViewValue(obj.value);
        $scope.input.value = obj.value;
        $scope.form[$scope.input.name].$render();
        th.hideTH();
    }

    function showSuggestion(obj) {
        if (!th.useSuggestionTemplae) {
            return obj.value;
        } else {
            return th.suggestionTemplate(obj);
        }
    }

    function hideTH() {
        $timeout(function() {
            th.showing = false;
        }, 200);
    }

    function inputExists() {
        return $scope.form[$scope.input.name].$modelValue != undefined;
    }
}

//Responsible for addition and removal of highlights
function ffTypeaheadHoover($log) {
    var directive = {
        restrict: 'A',
        link: linkFunction
    };
    return directive;

    function linkFunction(scope, el, attr) {
        el.bind('mouseenter', function() {
            el.css({'background-color': '#3caae6',
                    'cursor': 'pointer'});
        });
        el.bind('mouseleave', function() {
            el.css({'background-color': '#ffffff',
                    'cursor': 'pointer'
            });
        });
    }
}
