(function() {
    "use strict";
    angular.module("formFactory", [ "ui.router", "formFactory.dataFactory", "datatables", "datatables.bootstrap", "i18n", "ui.bootstrap", "ui.bootstrap.tooltip", "formFactory.commonUseFactory", "ngResource", "checklist-model", "uiSwitch", "formFactory.templateResolver", "toaster", "ngAnimate" ], function($uibTooltipProvider) {
        $uibTooltipProvider.options({
            placement: "bottom",
            popupDelay: 500,
            popupCloseDelay: 0,
            appendToBody: true
        });
    });
})();

(function() {
    var CaptchaController = function($scope, $log, $state, ffDataFactory, contextualData, toaster, i18n, $sce) {
        var cc = this;
        cc.captchaEnabled = false;
        cc.apiKey = null;
        cc.saveCaptcha = saveCaptcha;
        cc.captchaUnavailableMessage = $sce.trustAsHtml(i18n.format("ff.label.captcha.unavailableMessage", "<a href='https://www.google.com/recaptcha' target='_blank'>Google reCaptcha</a>"));
        $scope.$watch(function() {
            return cc.captchaEnabled;
        }, function(a, b) {
            if (a !== b && !cc.captchaEnabled) {
                var request = {
                    url: contextualData.context + "/modules/formfactory/settings/captchaapi/" + contextualData.siteIdentifier + "/en",
                    method: "DELETE"
                };
                if (cc.apiKey !== null && cc.apiKey.trim() !== "") {
                    ffDataFactory.customRequest(request).then(function(success) {
                        cc.apiKey = null;
                        toaster.pop({
                            type: "success",
                            title: i18n.message("ff.label.captcha.deleted")
                        });
                    }, function(error) {
                        console.log(error);
                    });
                }
            }
        });
        activate();
        function activate() {
            cc.apiKey = contextualData.captchaApiKey;
            if (cc.apiKey !== null && cc.apiKey !== "") {
                cc.captchaEnabled = true;
            }
        }
        function saveCaptcha() {
            var request = {
                url: contextualData.context + "/modules/formfactory/settings/captchaapi/" + contextualData.siteIdentifier + "/en",
                method: "POST",
                data: cc.apiKey
            };
            ffDataFactory.customRequest(request).then(function(success) {
                toaster.pop({
                    type: "success",
                    title: i18n.message("ff.label.captcha.saved")
                });
            }, function(error) {
                console.log(error);
            });
        }
    };
    CaptchaController.$inject = [ "$scope", "$log", "$state", "ffDataFactory", "contextualData", "toaster", "i18nService", "$sce" ];
    var captchaDirective = function($log, contextualData) {
        var directive = {
            restrict: "E",
            templateUrl: contextualData.context + "/modules/form-factory-core/javascript/angular/components/settings/captcha.html",
            controller: CaptchaController,
            controllerAs: "cc",
            bindToController: true,
            link: linkFunction
        };
        return directive;
        function linkFunction(scope, el, attr) {}
    };
    angular.module("formFactory").directive("ffCaptcha", [ "$log", "contextualData", captchaDirective ]);
})();

(function() {
    var mainSettings = function(contextualData) {
        return {
            restrict: "E",
            scope: {},
            templateUrl: contextualData.context + "/modules/form-factory-core/javascript/angular/components/settings/mainSettings.html",
            controller: MainSettingsController,
            controllerAs: "msc",
            bindToController: true,
            link: linkFunc
        };
        function linkFunc(scope, el, attr, ctrl) {}
    };
    angular.module("formFactory").directive("ffMainSettings", [ "contextualData", mainSettings ]);
    MainSettingsController.$inject = [ "contextualData", "ffDataFactory", "toaster", "i18nService" ];
    function MainSettingsController(contextualData, ffDataFactory, toaster, i18n) {
        var msc = this;
        msc.clearDefinitionsCache = clearDefinitionsCache;
        function clearDefinitionsCache() {
            var request = {
                url: contextualData.context + "/modules/formfactory/settings/flush/",
                method: "POST"
            };
            ffDataFactory.customRequest(request).then(function() {
                toaster.pop({
                    type: "success",
                    title: i18n.message("angular.ffMainSettingsDirective.title.flushSuccessful"),
                    toastId: "mscfs",
                    timeout: 3e3
                });
            }, function() {
                toaster.pop({
                    type: "error",
                    title: i18n.message("angular.ffMainSettingsDirective.title.flushFailed"),
                    toastId: "mscff",
                    timeout: 3e3
                });
            });
        }
    }
})();

angular.module("formFactory").config(function(contextualData, $stateProvider, $urlRouterProvider) {
    $urlRouterProvider.otherwise(function($injector, $location) {
        var $state = $injector.get("$state");
        if ($location.$$state == null) {
            $state.go("management", {}, {
                location: false
            });
        }
    });
    $stateProvider.state("management", {
        url: "/management",
        templateUrl: contextualData.urlBaseSiteSettingsTemplates + "management.html.ajax"
    });
}, [ "contextualData", "$stateProvider", "$urlRouterProvider" ]);

(function() {
    var CommonUseFactory = function($http, contextualData, $log) {
        return {
            checkIfNameExists: checkFormAvailability,
            logicResolver: logicResolver,
            valueResolver: valueResolver,
            placeholderResolver: placeholderResolver,
            inputsizeResolver: inputsizeResolver,
            nameResolver: nameResolver,
            labelResolver: labelResolver,
            parseDate: parseDate,
            createMoment: createMoment
        };
        function checkFormAvailability(newName, flag, formIdentifier) {
            var self = this;
            if (!_.isUndefined(newName) && newName != "") {
                var data = {
                    formName: newName,
                    siteIdentifier: contextualData.siteIdentifier
                };
                if (!_.isUndefined(formIdentifier)) {
                    data.formIdentifier = formIdentifier;
                }
                $http({
                    url: contextualData.context + "/modules/formfactory/builder/checkformnameavailability/" + contextualData.locale,
                    method: "POST",
                    data: data
                }).then(function(response) {
                    self[flag] = response.data.available;
                });
            } else {
                self[flag] = false;
            }
        }
        function valueResolver(args) {
            console.log("Resolving: ");
        }
        function placeholderResolver(input) {
            if ("placeholder" in input) {
                return input.placeholder;
            }
            return null;
        }
        function logicResolver(input, form) {
            if (input !== undefined) {
                if (!("ffVisible" in form[input.name])) {
                    return true;
                }
                return form[input.name].ffVisible;
            }
            return true;
        }
        function inputsizeResolver(input) {
            if ("inputsize" in input) {
                return input.inputsize;
            }
            return null;
        }
        function nameResolver(input) {
            if ("name" in input) {
                return input.name;
            }
            return null;
        }
        function labelResolver(input) {
            if ("label" in input) {
                return input.label;
            }
            return null;
        }
        function parseDate(value, format) {
            if (_.isUndefined(value) || value === null || value instanceof Date && isNaN(value.getTime())) {
                return false;
            }
            format = format != null ? format : "YYYY-MM-DD";
            if (value instanceof Date) {
                value = moment(value.toISOString()).format(format);
            } else if (moment.isMoment(value)) {
                return true;
            }
            if (typeof value === "string") {
                var components = value.split("-");
                if (components.length === 3) {
                    components = components[2].split(":");
                    if (components.length === 3) {
                        components = components[2].split(".");
                        if (components.length === 2) {
                            return true;
                        }
                    }
                }
            }
            return validateFormat(value, [ "-", "/" ], format).isValid;
        }
        function createMoment(value, formatSeparators, format) {
            if (_.isUndefined(value) || value === null) {
                return null;
            }
            if (value instanceof Date) {
                return moment(value.toISOString());
            } else if (moment(value, moment.ISO_8601, true).isValid()) {
                return moment(value, moment.ISO_8601, true).toISOString();
            }
            format = format != null ? format : "YYYY-MM-DD";
            var formatSeparators = _.isUndefined(formatSeparators) || formatSeparators === null ? [ "-", "/" ] : _.isString(formatSeparators) ? [].push(formatSeparators) : formatSeparators;
            var validatedDate = validateFormat(value, formatSeparators, format);
            return validatedDate.isValid ? validatedDate.date.toISOString() : null;
        }
        function validateFormat(value, formatSeparators, format) {
            var validatedDate = {
                isValid: false,
                date: null
            };
            for (var i in formatSeparators) {
                var sections = value.toString().split(formatSeparators[i]);
                var formatSections = format.split(formatSeparators[i]);
                if (sections.length == formatSections.length) {
                    validatedDate.isValid = true;
                    var formatSectionIndexes = {
                        year: null,
                        month: null,
                        date: null
                    };
                    for (var j in sections) {
                        if (sections[j] === undefined || sections[j] === "" || sections[j] === null || isNaN(sections[j]) || parseInt(sections[j]) < 0 || sections[j].toString().length !== formatSections[j].length) {
                            validatedDate.isValid = false;
                            break;
                        }
                        if (formatSections[j].toLowerCase().indexOf("y") !== -1) {
                            formatSectionIndexes.year = j;
                        } else if (formatSections[j].toLowerCase().indexOf("m") !== -1) {
                            formatSectionIndexes.month = j;
                        } else if (formatSections[j].toLowerCase().indexOf("d") !== -1) {
                            formatSectionIndexes.date = j;
                        }
                        sections[j] = parseInt(sections[j], 10);
                    }
                    if (!validatedDate.isValid) {
                        break;
                    }
                    validatedDate.date = moment();
                    if (!_.isNull(formatSectionIndexes.year)) {
                        validatedDate.date.year(sections[formatSectionIndexes.year]);
                    }
                    if (!_.isNull(formatSectionIndexes.month)) {
                        validatedDate.date.month(sections[formatSectionIndexes.month] - 1);
                    }
                    if (!_.isNull(formatSectionIndexes.date)) {
                        validatedDate.date.date(sections[formatSectionIndexes.date]);
                    }
                    var formatString = "";
                    for (var sec in formatSections) {
                        formatString += formatSections[sec].substring(0, 1);
                    }
                    validatedDate.isValid = validatedDate.date.format(formatString) == sections.join("-");
                }
            }
            return validatedDate;
        }
    };
    CommonUseFactory.$inject = [ "$http", "contextualData", "$log" ];
    angular.module("formFactory.commonUseFactory", []).factory("ffCommonUseFactory", CommonUseFactory);
})();

(function() {
    var dataFactory = function($http, $log) {
        return {
            getData: getData,
            customRequest: customRequest
        };
        function getData(url) {
            return $http.get(url).then(getDataComplete).catch(getDataFailed);
        }
        function customRequest(obj) {
            return $http(obj).then(getDataComplete);
        }
        function getDataComplete(response) {
            return response.data;
        }
        function getDataFailed(error) {
            $log.error("XHR Failed to execute your request." + error.data);
            return error.data;
        }
    };
    dataFactory.$inject = [ "$http", "$log" ];
    angular.module("formFactory.dataFactory", []).factory("ffDataFactory", dataFactory);
})();

(function() {
    var parseDate = function($log, ffCommonUseFactory, $timeout) {
        var directive = {
            restrict: "A",
            require: [ "^ngModel" ],
            link: linkFunction
        };
        return directive;
        function linkFunction(scope, el, attr, ctrls) {
            var model = ctrls[0];
            var hasParserExecuted = false;
            var parserInProgress = false;
            var viewValue;
            var AVAILABLE_FORMAT_SEPARATORS = [ "-", "/" ];
            var FORMAT_TEMPLATE = attr.format != null ? attr.format : "YYYY-MM-DD";
            var MAX_CHARACTERS = getMaxCharacters(FORMAT_TEMPLATE);
            var FORMAT_SEPARATOR_PROVIDED = !_.isUndefined(attr.ffParseDate) && attr.ffParseDate !== null && attr.ffParseDate.trim().length > 0;
            var REGEXP_FORMAT_SEPARATOR = FORMAT_SEPARATOR_PROVIDED ? attr.ffParseDate : "-|/";
            var FORMAT_REGEXP_PATTERN = "(^[0-9]+)(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "](?=[0-9]+)[0-9]+(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "](?=[0-9]+)[0-9]+(?=[0-9]*)" + "|(^[0-9]+)(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "](?=[0-9]+)[0-9]+(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "]" + "|(^[0-9]+)(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "](?=[0-9]+)[0-9]+" + "|(^[0-9]+)(?=[" + REGEXP_FORMAT_SEPARATOR + "]+)[" + REGEXP_FORMAT_SEPARATOR + "]" + "|(^[0-9]+)";
            var FORMAT_REGEXP = RegExp(FORMAT_REGEXP_PATTERN, "g");
            var REPLACE_REGEXP = RegExp(REGEXP_FORMAT_SEPARATOR, "g");
            var initializeViewValue = scope.$watch(function() {
                return model.$viewValue;
            }, function(value) {
                if (hasParserExecuted) {
                    initializeViewValue();
                    return;
                }
                viewValue = value;
                initializeViewValue();
            });
            function getFormattedRepresentation(value, formatSeparator) {
                if (value instanceof Date) {
                    return moment(value.toISOString(), moment.ISO_8601, true).format(FORMAT_TEMPLATE).replace(/-/g, formatSeparator);
                } else if (typeof value === "string" && moment(value, moment.ISO_8601, true).isValid()) {
                    return moment(value, moment.ISO_8601, true).format(FORMAT_TEMPLATE).replace(/-/g, formatSeparator);
                } else {
                    return value;
                }
            }
            function getFormatSeparator(value) {
                if (FORMAT_SEPARATOR_PROVIDED) {
                    return REGEXP_FORMAT_SEPARATOR;
                } else {
                    var formatSeparator = "-";
                    if (!_.isUndefined(value) && value !== null && !(value instanceof Date)) {
                        var separatorIndex = -1;
                        for (var i in AVAILABLE_FORMAT_SEPARATORS) {
                            var currentSeparatorIndex = value.indexOf(AVAILABLE_FORMAT_SEPARATORS[i]);
                            if (currentSeparatorIndex !== -1 && (separatorIndex === -1 || currentSeparatorIndex < separatorIndex)) {
                                separatorIndex = currentSeparatorIndex;
                                formatSeparator = AVAILABLE_FORMAT_SEPARATORS[i];
                            }
                        }
                    }
                    return formatSeparator;
                }
            }
            function setViewValueFromDateOrMoment(value, separator) {
                var m = value instanceof Date ? moment(viewValue.toISOString(), moment.ISO_8601, true) : moment(viewValue, moment.ISO_8601, true);
                value = m.format(FORMAT_TEMPLATE);
                if (separator !== "-") {
                    value.replace(/-/g, separator);
                }
                model.$setViewValue(value);
                model.$render();
                return m.toISOString();
            }
            function getMaxCharacters(format) {
                var maxCharacters;
                for (var i in AVAILABLE_FORMAT_SEPARATORS) {
                    if (format.indexOf(AVAILABLE_FORMAT_SEPARATORS[i]) !== -1) {
                        var sections = format.split(AVAILABLE_FORMAT_SEPARATORS[i]);
                        maxCharacters = sections.length - 1;
                        for (var j in sections) {
                            maxCharacters += sections[j].length;
                        }
                        break;
                    }
                }
                return maxCharacters;
            }
            model.$parsers.push(function(value) {
                hasParserExecuted = true;
                if (parserInProgress) {
                    return value;
                }
                parserInProgress = true;
                if (_.isUndefined(value) || value === null) {
                    parserInProgress = false;
                    viewValue = value;
                    return value;
                }
                if (value instanceof Date) {
                    model.$setViewValue(getFormattedRepresentation(value, getFormatSeparator(viewValue)));
                    viewValue = value;
                    model.$render();
                    parserInProgress = false;
                    return moment(value.toISOString(), moment.ISO_8601, true).toISOString();
                }
                var matchedValue = value.match(FORMAT_REGEXP);
                if (matchedValue === null) {
                    value = viewValue;
                    model.$setViewValue(getFormattedRepresentation(value, getFormatSeparator(value)));
                    model.$render();
                    parserInProgress = false;
                    return value;
                }
                var formatSeparator = getFormatSeparator(value);
                value = viewValue !== null && matchedValue[0].length < viewValue.length && matchedValue[0].length != value.length ? viewValue : matchedValue[0];
                value = value.replace(REPLACE_REGEXP, formatSeparator);
                if (value.toString().length > MAX_CHARACTERS) {
                    if (ffCommonUseFactory.parseDate(value, FORMAT_TEMPLATE)) {
                        var momentFormat = formatSeparator !== "-" ? angular.copy(FORMAT_TEMPLATE).replace(/-/g, formatSeparator) : FORMAT_TEMPLATE;
                        var m = moment(ffCommonUseFactory.createMoment(value, FORMAT_SEPARATOR_PROVIDED ? REGEXP_FORMAT_SEPARATOR : [ "-", "/" ], FORMAT_TEMPLATE), moment.ISO_8601, true);
                        value = m.toISOString();
                        viewValue = value;
                        model.$setViewValue(m.format(momentFormat));
                        model.$render();
                    } else {
                        if (viewValue instanceof Date || moment(viewValue, moment.ISO_8601, true).isValid()) {
                            value = setViewValueFromDateOrMoment(viewValue, formatSeparator);
                        } else {
                            value = viewValue;
                            model.$setViewValue(value);
                            model.$render();
                            value = moment(value, angular.copy(FORMAT_TEMPLATE).replace(/-/g, formatSeparator), true).toISOString();
                        }
                    }
                } else {
                    viewValue = value;
                    model.$setViewValue(value);
                    model.$render();
                    if (ffCommonUseFactory.parseDate(value, FORMAT_TEMPLATE)) {
                        value = moment(value, angular.copy(FORMAT_TEMPLATE).replace(/-/g, formatSeparator)).toISOString();
                    }
                }
                $timeout(function() {
                    model.$setValidity("date", true);
                });
                parserInProgress = false;
                return value;
            });
        }
    };
    angular.module("formFactory").directive("ffParseDate", [ "$log", "ffCommonUseFactory", "$timeout", parseDate ]);
})();

(function() {
    "use strict";
    var datePicker = function($log, contextualData) {
        var directive = {
            restrict: "E",
            scope: {
                format: "@?",
                minValue: "=?",
                maxValue: "=?",
                datePickerSize: "=",
                minDatePlaceholder: "=",
                maxDatePlaceholder: "=",
                datePickerOptions: "=?",
                datePickerMode: "@?",
                minLabel: "@?",
                maxLabel: "@?",
                readOnly: "@?",
                isDisabled: "@?",
                enforceMinValue: "@?",
                enforceMaxValue: "@?",
                errorControl: "=?",
                parserSeparator: "@?"
            },
            controller: DatePickerController,
            controllerAs: "dpc",
            bindToController: true,
            templateUrl: contextualData.context + "/modules/form-factory-core/javascript/angular/components/datePicker.html",
            link: linkFunc
        };
        return directive;
        function linkFunc(scope, el, attr, ctrl) {}
    };
    angular.module("formFactory").directive("ffDatePicker", [ "$log", "contextualData", datePicker ]);
    var DatePickerController = function($scope, $log, i18n, uibDateParser, $timeout, ffCommonUseFactory) {
        var dpc = this;
        var DEFAULT_FORMAT = "YYYY-MM-DD";
        dpc.isOpenMin = false;
        dpc.isOpenMax = false;
        dpc.previousValue = {
            min: "",
            max: ""
        };
        dpc.selectedMinDate;
        dpc.selectedMaxDate;
        dpc.openDatePicker = openDatePicker;
        dpc.executeOnChange = executeOnChange;
        dpc.i18nMessageGetter = i18n.message;
        dpc.format = dpc.format == null || dpc.format.trim().length == 0 ? DEFAULT_FORMAT : dpc.format;
        dpc.uibDateParserFormat = convertFormatToUibDateFormat(dpc.format);
        activate();
        function activate() {
            dpc.isAvailableDatepicker = {
                min: _.isUndefined(dpc.minValue) ? false : true,
                max: _.isUndefined(dpc.maxValue) ? false : true
            };
            if (_.isObject(dpc.errorControl)) {
                if (!_.isUndefined(dpc.errorControl.min) && _.isObject(dpc.errorControl.min)) {
                    dpc.errorControl.min.hasError = !_.isUndefined(dpc.errorControl.min.hasError) && _.isBoolean(dpc.errorControl.min.hasError) ? dpc.errorControl.min.hasError : false;
                } else {
                    dpc.errorControl.min = {
                        hasError: false,
                        errorMessage: null
                    };
                }
                if (!_.isUndefined(dpc.errorControl.max) && _.isObject(dpc.errorControl.max)) {
                    dpc.errorControl.max.hasError = !_.isUndefined(dpc.errorControl.max.hasError) && _.isBoolean(dpc.errorControl.max.hasError) ? dpc.errorControl.max.hasError : false;
                } else {
                    dpc.errorControl.max = {
                        hasError: false,
                        errorMessage: null
                    };
                }
            } else {
                dpc.errorControl = {
                    min: {
                        hasError: false,
                        errorMessage: null
                    },
                    max: {
                        hasError: false,
                        errorMessage: null
                    }
                };
            }
            dpc.minLabelAvailable = !_.isUndefined(dpc.minLabel);
            dpc.maxLabelAvailable = !_.isUndefined(dpc.maxLabel);
            dpc.readOnly = _.isUndefined(dpc.readOnly) ? false : dpc.readOnly === true || dpc.readOnly == "true";
            dpc.isDisabled = _.isUndefined(dpc.isDisabled) ? false : dpc.isDisabled === true || dpc.isDisabled == "true";
            dpc.enforceMinValue = _.isUndefined(dpc.enforceMinValue) ? false : dpc.enforceMinValue === true || dpc.enforceMinValue == "true";
            dpc.enforceMaxValue = _.isUndefined(dpc.enforceMaxValue) ? false : dpc.enforceMaxValue === true || dpc.enforceMaxValue == "true";
            setupDatepickerOptions();
            if (!_.isUndefined(dpc.minValue)) {
                if (dpc.minValue instanceof Date) {
                    dpc.selectedMinDate = dpc.minValue;
                } else if (moment(dpc.minValue, moment.ISO_8601, true).isValid()) {
                    dpc.selectedMinDate = new Date(moment(dpc.minValue, moment.ISO_8601, true).toISOString());
                } else {
                    var verifyDate = new Date(dpc.minValue);
                    if (dpc.minValue !== null && verifyDate instanceof Date && !isNaN(verifyDate.getTime())) {
                        dpc.selectedMinDate = verifyDate;
                    } else {
                        dpc.selectedMinDate = null;
                    }
                }
                dpc.initMin = !_.isUndefined(dpc.datePickerOptions.maxDate) && moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).isBefore(new Date().toISOString()) ? moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).toDate() : new Date();
            }
            if (!_.isUndefined(dpc.maxValue)) {
                if (dpc.maxValue instanceof Date) {
                    dpc.selectedMaxDate = new Date(dpc.maxValue.toISOString());
                } else if (moment(dpc.maxValue, moment.ISO_8601, true).isValid()) {
                    dpc.selectedMaxDate = new Date(moment(dpc.maxValue, moment.ISO_8601, true).toISOString());
                } else {
                    var verifyDate = new Date(dpc.maxValue);
                    if (dpc.maxValue !== null && verifyDate instanceof Date && !isNaN(verifyDate.getTime())) {
                        dpc.selectedMaxDate = verifyDate;
                    } else {
                        dpc.selectedMaxDate = null;
                    }
                }
                dpc.initMax = !_.isUndefined(dpc.datePickerOptions.maxDate) && moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).isBefore(new Date().toISOString()) ? moment(dpc.datePickerOptions.maxDate, moment.ISO_8601, true).toDate() : new Date();
            }
            dpc.previousValue.min = angular.copy(dpc.selectedMinDate);
            dpc.previousValue.max = angular.copy(dpc.selectedMaxDate);
        }
        function openDatePicker(popup) {
            dpc.isOpenMin = popup === "min";
            dpc.isOpenMax = popup === "max";
            if (dpc.isOpenMin) {
                $timeout(function() {
                    $timeout(function() {
                        angular.element("div.datePickerMin").find("ul.uib-datepicker-popup button.uib-datepicker-current").focus();
                        angular.element("div.datePickerMin").find("ul.uib-datepicker-popup button.uib-datepicker-current").blur();
                        angular.element("div.datePickerMin").find("table").focus();
                    });
                });
            } else if (dpc.isOpenMax) {
                $timeout(function() {
                    $timeout(function() {
                        angular.element("div.datePickerMax").find("ul.uib-datepicker-popup button.uib-datepicker-current").focus();
                        angular.element("div.datePickerMax").find("ul.uib-datepicker-popup button.uib-datepicker-current").blur();
                        angular.element("div.datePickerMax").find("table").focus();
                    });
                });
            }
        }
        function setupDatepickerOptions() {
            if (_.isUndefined(dpc.datePickerOptions)) {
                dpc.datePickerOptions = {};
                return;
            } else {
                _.each(dpc.datePickerOptions, function(option, optionName) {
                    if (_.isUndefined(option) || _.isEmpty(option)) {
                        delete dpc.datePickerOptions[optionName];
                    }
                });
            }
        }
        function executeOnChange(input) {
            if (dpc.enforceMinValue && input === "min" && (_.isUndefined(dpc.selectedMinDate) || !moment(dpc.selectedMinDate, moment.ISO_8601, true).isValid())) {
                dpc.selectedMinDate = moment("1970", "YYYY").toDate();
            }
            if (dpc.enforceMaxValue && input === "max" && (_.isUndefined(dpc.selectedMaxDate) || !moment(dpc.selectedMaxDate, moment.ISO_8601, true).isValid())) {
                dpc.selectedMaxDate = moment().toDate();
            }
            if (input === "min" && ffCommonUseFactory.parseDate(dpc.selectedMinDate, dpc.format) && !_.isUndefined(dpc.selectedMaxDate) && ffCommonUseFactory.parseDate(dpc.selectedMaxDate, dpc.format)) {
                if (moment(dpc.selectedMinDate).isAfter(moment(dpc.selectedMaxDate).toISOString())) {
                    dpc.selectedMinDate = uibDateParser.parse(new Date(dpc.previousValue.min));
                }
            }
            if (input === "max" && ffCommonUseFactory.parseDate(dpc.selectedMaxDate, dpc.format) && !_.isUndefined(dpc.selectedMinDate) && ffCommonUseFactory.parseDate(dpc.selectedMinDate, dpc.format)) {
                if (moment(dpc.selectedMaxDate).isBefore(moment(dpc.selectedMinDate).toISOString())) {
                    dpc.selectedMaxDate = uibDateParser.parse(new Date(dpc.previousValue.max));
                }
            }
            if (input === "min" && ffCommonUseFactory.parseDate(dpc.selectedMinDate, dpc.format)) {
                dpc.previousValue.min = angular.copy(dpc.selectedMinDate);
            }
            if (input === "max" && ffCommonUseFactory.parseDate(dpc.selectedMaxDate, dpc.format)) {
                dpc.previousValue.max = angular.copy(dpc.selectedMaxDate);
            }
            if (input === "min" && !_.isUndefined(dpc.selectedMinDate)) {
                $timeout(function() {
                    $scope.$apply(function() {
                        dpc.minValue = dpc.selectedMinDate;
                    });
                });
            }
            if (input === "max" && !_.isUndefined(dpc.selectedMaxDate)) {
                $timeout(function() {
                    $scope.$apply(function() {
                        dpc.maxValue = dpc.selectedMaxDate;
                    });
                });
            }
        }
        function convertFormatToUibDateFormat(format) {
            if (format == null || format.trim().length == 0) {
                format = DEFAULT_FORMAT;
            }
            var separator = dpc.parserSeparator != null && dpc.parserSeparator.trim().length > 0 ? [ dpc.parserSeparator ] : [ "-", "/" ];
            var uibFormat = "";
            var separatorDetected = false;
            for (var i in separator) {
                if (format.indexOf(separator[i]) !== -1) {
                    var sections = format.split(separator[i]);
                    for (var j in sections) {
                        if (sections[j].toLowerCase().indexOf("y") !== -1) {
                            uibFormat += sections[j].toLowerCase() + (j < sections.length - 1 ? separator[i] : "");
                        } else if (sections[j].toLowerCase().indexOf("m") !== -1) {
                            uibFormat += sections[j].toUpperCase() + (j < sections.length - 1 ? separator[i] : "");
                        } else if (sections[j].toLowerCase().indexOf("d") !== -1) {
                            uibFormat += sections[j].toLowerCase() + (j < sections.length - 1 ? separator[i] : "");
                        }
                    }
                    separatorDetected = true;
                    break;
                }
            }
            if (!separatorDetected) {
                if (format.toLowerCase().indexOf("y") !== -1) {
                    uibFormat += format.toLowerCase();
                } else if (format.toLowerCase().indexOf("m") !== -1) {
                    uibFormat += format.toUpperCase();
                } else if (format.toLowerCase().indexOf("d") !== -1) {
                    uibFormat += format.toLowerCase();
                }
            }
            return uibFormat;
        }
    };
    DatePickerController.$inject = [ "$scope", "$log", "i18nService", "uibDateParser", "$timeout", "ffCommonUseFactory" ];
    var dateValidator = function($log, ffCommonUseFactory) {
        var directive = {
            restrict: "A",
            require: [ "^ngModel" ],
            link: linkFunc
        };
        return directive;
        function linkFunc(scope, el, attr, ctrl) {
            var model = ctrl[0];
            var fieldId = attr.fieldId;
            var caseName = attr.caseName;
            model.$validators[attr.ffDateValidator] = function() {
                var isValid = ffCommonUseFactory.parseDate(model.$viewValue);
                scope.$emit("requiredValueChanged", {
                    isValid: isValid,
                    fieldId: fieldId,
                    caseName: caseName,
                    validator: attr.ffDateValidator
                });
                return isValid;
            };
        }
    };
    angular.module("formFactory").directive("ffDateValidator", [ "$log", "ffCommonUseFactory", dateValidator ]);
})();

(function() {
    var editInPlace = function() {
        var directive = {
            restrict: "E",
            scope: {
                value: "="
            },
            template: '<span ng-click="edit()" ng-bind="value"></span><input class="form-control" ng-model="value" />',
            link: linkFunction
        };
        return directive;
        function linkFunction(scope, el, attrs) {
            var inputElement = el.find("input");
            el.addClass("edit-in-place");
            el.addClass("inputHidden");
            scope.edit = function() {
                el.addClass("spanHidden");
                el.removeClass("inputHidden");
                inputElement[0].focus();
            };
            inputElement.on("blur", function(e) {
                e.stopPropagation();
                el.removeClass("spanHidden");
                el.addClass("inputHidden");
            });
        }
    };
    angular.module("formFactory").directive("ffEditInPlace", [ editInPlace ]);
})();

(function() {
    "use strict";
    var logicsFieldValidation = function($log, contextualData, $timeout) {
        var directive = {
            restrict: "EA",
            require: [ "^ngModel" ],
            link: linkFunc
        };
        return directive;
        function linkFunc(scope, el, attr, ctrl) {
            var type = attr["ffFieldValueValidation"];
            var fieldId = attr["fieldId"];
            var caseName = attr["caseName"];
            el.bind("keyup", function(e) {
                scope.$apply(function() {
                    validate(e.currentTarget.value);
                });
            });
            el.bind("change", function(e) {
                scope.$apply(function() {
                    validate(e.currentTarget.value);
                });
            });
            $timeout(function() {
                validate(ctrl[0].$viewValue);
            }, 500);
            function validate(value) {
                switch (type) {
                  case "any":
                    if (value === undefined || value === "" || value === null) {
                        scope.$emit("requiredValueChanged", {
                            isValid: false,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    } else {
                        scope.$emit("requiredValueChanged", {
                            isValid: true,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    }
                    break;

                  case "number":
                    if (value === undefined || value === "" || value === null || isNaN(value)) {
                        scope.$emit("requiredValueChanged", {
                            isValid: false,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    } else {
                        scope.$emit("requiredValueChanged", {
                            isValid: true,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    }
                    break;

                  case "positiveNumber":
                    if (value === undefined || value === "" || value === null || isNaN(value) || value < 0) {
                        scope.$emit("requiredValueChanged", {
                            isValid: false,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    } else {
                        scope.$emit("requiredValueChanged", {
                            isValid: true,
                            "case": type,
                            fieldId: fieldId,
                            caseName: caseName
                        });
                    }
                    break;

                  case "multiple":
                    scope.$emit("requiredValueChanged", {
                        isValid: scope.vsc.selectedValues.length > 0,
                        "case": type,
                        fieldId: fieldId,
                        caseName: caseName
                    });
                    break;
                }
            }
        }
    };
    angular.module("formFactory").directive("ffFieldValueValidation", [ "$log", "contextualData", "$timeout", logicsFieldValidation ]);
})();

(function() {
    "use strict";
    var ResolveTemplatesService = function($log) {
        this.resolveTemplatePath = function(path, viewType) {
            viewType = viewType !== undefined ? "." + viewType : "";
            return path.replace(".html.ajax", viewType + ".html.ajax");
        };
    };
    ResolveTemplatesService.$inject = [ "$log" ];
    angular.module("formFactory.templateResolver", []).service("ffTemplateResolver", ResolveTemplatesService);
})();

(function() {
    "use strict";
    var sanitize = function($sce) {
        return function(htmlCode) {
            return $sce.trustAsHtml(htmlCode);
        };
    };
    var removeSpaces = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return string.replace(/[\s]/g, "");
        };
    };
    var lowercase = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return string.toLowerCase();
        };
    };
    var normalize = function() {
        return function(value) {
            var newValue = "";
            for (var i = 0; i < value.length; i++) {
                if (value.charAt(i) === "-") {
                    if (i++ < value.length) {
                        newValue += value.charAt(i).toUpperCase();
                    }
                    continue;
                }
                newValue += value.charAt(i);
            }
            return newValue;
        };
    };
    var removeDashes = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return string.replace(/[\-]/g, "");
        };
    };
    var callUnderscoreString = function() {
        return function(str, fn, params) {
            str = str || "";
            params = params || [];
            params.unshift(str);
            return fn ? s[fn].apply(this, params) : str;
        };
    };
    var momentCalendar = function() {
        return function(string) {
            if (!angular.isString(string)) {
                return string;
            }
            return moment(string, "LLL").calendar();
        };
    };
    var resourceBundle = function(i18n) {
        return function(string, resourcePartial) {
            return i18n.message(resourcePartial + "." + string);
        };
    };
    var removeChar = function() {
        return function(string, charIndex) {
            return string.substr(charIndex);
        };
    };
    var sanitizeFormName = function() {
        return function(formName) {
            if (!angular.isString(formName)) {
                return formName;
            }
            return formName.replace(/[`\s\'\"~+=!*&%#@,:;\.\^\$\?<\>\|\/\\\[\]\(\)\}\{\-]/g, "");
        };
    };
    var roundValue = function() {
        return function(value) {
            var regexp = new RegExp(/^-{0,1}\d*\.?\d*$/);
            if (!regexp.test(value)) {
                return value;
            }
            return Math.round(value);
        };
    };
    var dasherize = function() {
        return function(str) {
            return str != null ? str.toLowerCase().replace(/\s/g, "-") : "";
        };
    };
    angular.module("formFactory").filter("sanitize", [ "$sce", sanitize ]).filter("removeSpaces", [ removeSpaces ]).filter("lowercase", [ lowercase ]).filter("normalize", [ normalize ]).filter("removeDashes", [ removeDashes ]).filter("s", [ callUnderscoreString ]).filter("momentCalendar", [ momentCalendar ]).filter("i18n", [ "i18nService", resourceBundle ]).filter("removeIndex", [ removeChar ]).filter("sanitizeFormName", [ sanitizeFormName ]).filter("roundValue", [ roundValue ]).filter("dasherize", [ dasherize ]);
})();

(function() {
    "use strict";
    var i18nMessageKeyDirectiveFunction = function(i18nService) {
        return {
            restrict: "A",
            link: function($scope, $element, $attrs) {
                var i18n;
                if (!$attrs.messageParams) {
                    i18n = i18nService.message($attrs.messageKey);
                } else {
                    i18n = i18nService.format($attrs.messageKey, $attrs.messageParams);
                }
                if ($attrs.messageAttr) {
                    $element.attr($attrs.messageAttr, i18n);
                } else {
                    $element.text(i18n);
                }
            }
        };
    };
    var i18nTranslateFilterFunction = function(i18nService) {
        return function(input) {
            return i18nService.message(input);
        };
    };
    angular.module("i18n", []).service("i18nService", function() {
        this.message = function(key, fallback) {
            if (ffi18n && ffi18n[key]) {
                return ffi18n[key];
            } else if (angular.isDefined(fallback)) {
                return fallback;
            } else {
                return "???" + key + "???";
            }
        };
        this.format = function(key, params) {
            var replacer = function(params) {
                return function(s, index) {
                    return params[index] ? params[index] === "__void__" ? "" : params[index] : "";
                };
            };
            if (params) {
                if (ffi18n && ffi18n[key]) {
                    return ffi18n[key].replace(/\{(\w+)\}/g, replacer(params.split("|")));
                } else {
                    return "???" + key + "???";
                }
            } else {
                return this.message(key);
            }
        };
    }).directive("messageKey", [ "i18nService", i18nMessageKeyDirectiveFunction ]).filter("translate", [ "i18nService", i18nTranslateFilterFunction ]);
})();

(function() {
    "use strict";
    var UnderscoreFactory = function() {
        return window._;
    };
    angular.module("formFactory").factory("_", UnderscoreFactory);
})();