package org.jahia.modules.formfactory.dsl

import org.jahia.api.Constants
import org.jahia.data.templates.JahiaTemplatesPackage
import org.jahia.services.content.*
import org.jahia.services.content.nodetypes.ExtendedNodeType
import org.jahia.services.templates.TemplatePackageRegistry
import org.jahia.services.usermanager.JahiaUserManagerService
import org.jahia.settings.SettingsBean
import org.slf4j.Logger
import org.slf4j.LoggerFactory

import javax.jcr.RepositoryException

class CallbackHandler implements DSLHandler {
    private static final Logger logger = LoggerFactory.getLogger(CallbackHandler.class);
    def JCRTemplate jcrTemplate;
    def TemplatePackageRegistry packageRegistry
    def SettingsBean settingsBean

    def callback(@DelegatesTo(FormFactoryWizardParser) Closure cl) {
        parseDeclaration(cl)
    }

    def parseDeclaration(Closure cl) {
        def callbackType = new FormFactoryWizardParser()
        def code = cl.rehydrate(callbackType, this, this)
        code.resolveStrategy = Closure.DELEGATE_FIRST
        code()
        logger.info("Declared callback structure: " + callbackType.toString());
        def result = callbackType.contentMap
        createWizard result, cl.owner.currentPackage, cl.owner.currentNodeType
    }

//    def registerProperties(wizardEntry, value) {
//        if (value instanceof String) {
//            wizardEntry.setProperty("jsonValue", value)
//        } else if (value instanceof Map) {
//            assert value["values"] != null
//
//            def values = value["values"]
//            if (values instanceof List) {
//                def jsonValue = []
//                values.eachWithIndex { label, idx ->
//                    jsonValue[idx] = [label: label, value: label.toString().toLowerCase(), selected: (idx == 0)]
//                }
//                def json = new JsonBuilder()
//                json jsonValue
//                wizardEntry.setProperty("jsonValue", json.toString())
//            } else if (values instanceof Map) {
//                def jsonValue = []
//                values.eachWithIndex { k, l, idx ->
//                    jsonValue[idx] = [key: k, value: l]
//                }
//                def json = new JsonBuilder()
//                json jsonValue
//                wizardEntry.setProperty("jsonValue", json.toString())
//            }
//        } else if (value instanceof List) {
//            wizardEntry.setProperty("jsonValue", value[1])
//        }
//    }

    def createWizard(Map<String, Object> map, JahiaTemplatesPackage currentPackage, ExtendedNodeType currentNodeType) {
        assert map.containsKey("label")
        jcrTemplate.doExecuteWithSystemSessionAsUser(JahiaUserManagerService.instance.lookupRootUser().jahiaUser, Constants.EDIT_WORKSPACE, Locale.ENGLISH, new JCRCallback() {
            @Override
            Object doInJCR(JCRSessionWrapper jcrSessionWrapper) throws RepositoryException {
                JCRNodeWrapper bundleRootNode = jcrSessionWrapper.getNode(currentPackage.rootFolderPath + "/" + currentPackage.getVersion().toString())
                def rNodeWrapper = bundleRootNode.getNode("templates").getNode("contents")
                if (!rNodeWrapper.hasNode("form-factory-callbacks")) {
                    rNodeWrapper.addNode("form-factory-callbacks","jnt:contentFolder")
                }
                def callbacksRootNode = rNodeWrapper.getNode("form-factory-callbacks")
                def nodeName = JCRContentUtils.generateNodeName(map["label"], 32)

                if (callbacksRootNode.hasNode(nodeName)) {
                    //Reintroduce check for development if necessary
                    /*if (settingsBean.isDevelopmentMode()) {
                        callbacksRootNode.getNode(nodeName).remove()
                        jcrSessionWrapper.save()
                    } else {
                        return null;
                    }*/
                    return null;
                }

                def node = callbacksRootNode.addNode(nodeName, currentNodeType.name)
                node.setProperty(Constants.JCR_TITLE, map["label"])
                map.remove("label")

                jcrSessionWrapper.save()
                return null
            }
        })
    }
}

